<?php

/**
 * Add All Access Passes to the EDD Customer Interface
 * *
 * @since 1.0.0
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) { exit; }

/**
 * Add the All Access Passes tab to the customer interface if the customer has All Access Passes
 *
 * @since  1.0.0
 * @param  array $tabs The tabs currently added to the customer view
 * @return array       Updated tabs array
 */
function edd_all_access_customer_tab( $tabs ) {

	// This makes it so former commission recievers get the tab and new commission users with no sales see it
	$tabs['all-access-passes'] = array( 'dashicon' => 'dashicons-welcome-widgets-menus', 'title' => __( 'All Access Passes', 'edd-all-access' ) );

	return $tabs;
}
add_filter( 'edd_customer_tabs', 'edd_all_access_customer_tab', 10, 1 );

/**
 * Register the All Access Passes view for the customer interface
 *
 * @since  1.0.0
 * @param  array $tabs The tabs currently added to the customer views
 * @return array       Updated tabs array
 */
function edd_all_access_add_customer_view( $views ) {

	$views['all-access-passes'] = 'edd_all_access_customer_view';

	return $views;
}
add_filter( 'edd_customer_views', 'edd_all_access_add_customer_view', 10, 1 );

/**
 * Display the All Access Passes area for the customer view
 *
 * @since  1.0.0
 * @param  object $customer The Customer being displayed
 * @return void
 */
function edd_all_access_customer_view( $customer ) {

	?>
	<div class="customer-notes-header">
		<?php echo get_avatar( $customer->email, 30 ); ?> <span><?php echo $customer->name; ?></span>
	</div>

	<div id="customer-tables-wrapper" class="customer-section">
		<?php

		// Get the All Access passes saved to this customer meta
		$customer_all_access_passes = $customer->get_meta( 'all_access_passes' );

		$at_least_one_pass_to_show = false;

		?>
		<h3><?php _e( 'All Access Passes', 'edd-all-access' ); ?></h3>
		<table class="wp-list-table widefat striped downloads">
			<thead>
				<tr>
					<th><?php echo edd_get_label_singular(); ?></th>
					<th><?php _e( 'Start Date', 'edd-all-access' ); ?></th>
					<th><?php _e( 'Expiration Date', 'edd-all-access' ); ?></th>
					<th><?php _e( 'Status', 'edd-all-access' ); ?></th>
					<th><?php _e( 'Actions', 'edd-all-access' ); ?></th>
				</tr>
			</thead>

			<tbody>
				<?php
				if ( is_array( $customer_all_access_passes )  ){

					foreach( $customer_all_access_passes as $purchased_download_id_price_id => $purchased_aa_data ){

							// In case there happens to be an entry in the array without a numeric key
							if ( empty( $purchased_download_id_price_id ) ){
								continue;
							}

							if ( ! isset( $purchased_aa_data['payment_id'] ) || ! isset( $purchased_aa_data['download_id'] ) || ! isset( $purchased_aa_data['price_id'] ) ){
								continue;
							}
							
							$all_access_pass = new EDD_All_Access_Pass( $purchased_aa_data['payment_id'], $purchased_aa_data['download_id'], $purchased_aa_data['price_id'] );

							if ( $all_access_pass->status == 'invalid' ){
								continue;
							}

							$at_least_one_pass_to_show = true;

							?>
							<tr>
								<td><a href="<?php echo esc_url( admin_url( 'post.php?action=edit&post=' . $purchased_aa_data['download_id'] ) ); ?>"><?php echo get_the_title( $purchased_aa_data['download_id']  ); ?></a></td>
								<td><?php echo date( 'M d, Y', $all_access_pass->start_time ); ?></td>
								<td><?php echo $all_access_pass->expiration_time == 'never' ? __( 'Never Expires', 'edd-all-access' ) : date( 'M d, Y', $all_access_pass->expiration_time ); ?></td>
								<td><?php echo edd_all_access_get_status_label( $all_access_pass->status ); ?></td>
								<td><a href="<?php echo esc_url( admin_url( 'edit.php?post_type=download&page=edd-all-access-pass&payment_id=' . $all_access_pass->payment->ID . '&download_id=' . $purchased_aa_data['download_id'] . '&price_id=' . $purchased_aa_data['price_id'] ) ); ?>"><?php _e( 'View Details', 'edd-all-access' ); ?></a>
							</tr><?php
					}
				}

				// If there was not at least 1 valid pass to show
				if ( ! $at_least_one_pass_to_show ){ ?>
					<tr><td colspan="5"><?php _e( 'No All Access Passes Found', 'edd-all-access' ); ?></td></tr>
				<?php } ?>
			</tbody>
		</table>

		<?php //If the EDD Dev Tools is active, output a little extra here
		if ( class_exists( 'EDD_Dev_Tools' ) ) {
			echo '<div class="edd-all-access-dev-tools-area" style="background-color:#fffbb4; padding:10px;" >';
			echo '<h2>' . __( 'Customer All Access Passes Array:', 'edd-all-access' ) . '</h2>';
			echo '<p>' . __( 'You are seeing this because you have EDD Dev Tools Activated', 'edd-all-access' ) . '</p>';
			echo '<pre>';
				print_r($customer_all_access_passes);
			echo '</pre>';
			echo '</div>';
		}?>

	</div><?php
}
