<?php
/**
 * Ajax Callback Functions
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Ajax callback for switching variable price IDs and putting the right download URL onto the "Download" button for All Access.
 *
 * @since    1.0.0
 * @param    void
 * @return   void
 */
function edd_all_access_update_download_url() {

	$download_id = ! empty( $_POST['edd_all_access_download_id'] ) ? absint( $_POST['edd_all_access_download_id'] ) : 0;
	$price_id    = ! empty( $_POST['edd_all_access_price_id'] ) ? absint( $_POST['edd_all_access_price_id'] ) : 0;

	$return_array = array(
		'success' => true,
		'button_html' => edd_get_purchase_link( $args = array(
			'download_id' => $download_id,
			'price_id' => $price_id,
		) ),
	);

	echo json_encode( $return_array );

	die();

}
add_action( 'wp_ajax_edd_all_access_update_download_url', 'edd_all_access_update_download_url' );
add_action( 'wp_ajax_nopriv_edd_all_access_update_download_url', 'edd_all_access_update_download_url' );

/**
 * Process All Access Payments via ajax
 *
 * @since 1.0.0
 * @return void
 */
function edd_all_access_do_ajax_process() {

	parse_str( $_POST['form'], $form );

	$_REQUEST = $form = (array) $form;

	if( ! wp_verify_nonce( $_REQUEST['edd_all_access_ajax_process'], 'edd_all_access_ajax_process' ) ) {
		die( __( 'Nonce Failure. Please try again.', 'edd-all-access' ) );
	}

	$step = absint( $_POST['step'] );

	// Get all product/downloads which are All Access-enabled
	$all_access_products = edd_all_access_get_all_access_downloads();

	// Get all payments in groups of 10 per step.
	$args = array(
		'post_type'      => 'edd_payment',
		'offset'         => 10 * $step, // Check 10 payments per step
		'posts_per_page' => 10,
	);

	// We would use EDD_Payments_Query but it doesn't give us the data we need like found_posts and also has some other bugs currently preventing proper usage:
	// For example: https://github.com/easydigitaldownloads/easy-digital-downloads/issues/5377
	$query = new WP_Query( $args );

	$payments_completed_in_this_step = array();

	// Loop through the 10 payments we just queried
	if ( $query->have_posts() ) {
		while ( $query->have_posts() ) {
			$query->the_post();
			$payment = new EDD_Payment( $query->post->ID );

			$payments_completed_in_this_step[] = $payment->ID;

			// Loop through all downloads purchased in this payment
			foreach( $payment->downloads as $purchased_download ){

				if ( in_array( $purchased_download['id'], $all_access_products ) ){
					// Set up the All Access Pass object
					$all_access_pass = new EDD_All_Access_Pass( $payment->ID, $purchased_download['id'], $purchased_download['options']['price_id'] );

					// Run a check which fixes incorrect meta data due to issue 152 on GitHub
					all_access_issue_152_check( $all_access_pass, $payment->ID, $purchased_download['id'], $purchased_download['options']['price_id'] );

					//Attempt to activate the All Access Pass.
					$all_access_pass->maybe_activate();

					//Attempt to expire the All Access Pass.
					$all_access_pass->maybe_expire();

				}

				$payment->add_note( __( 'This payment was processed by the All Access processing tool.', 'edd-all-access' ) );
			}
		}

		if ( $query->found_posts <= 10 ){
			$percentage = 100;
		}else{
			$percentage = ( ($step*10) / $query->found_posts ) * 100;
		}

		$step += 1;
		echo json_encode( array( 'step' => $step, 'percentage' => $percentage, 'payments_completed_in_this_step' => $payments_completed_in_this_step ) ); exit;

	}else{
		//If no more Payments are left
		$message = __( 'Batch Processing Complete', 'edd-all-access' );
		echo json_encode( array( 'success' => true, 'message' => $message ) ); exit;

	}

}
add_action( 'wp_ajax_edd_all_access_do_ajax_process', 'edd_all_access_do_ajax_process' );

/**
 * Check if a being-updated All Access would would be set to expire so we know whether to show the "Are you sure" popup.
 *
 * @since 1.0.0
 * @return void
 */
function edd_all_access_expiration_check(){

	$would_be_duration_number = $_POST['duration_number'];
	$would_be_duration_unit = $_POST['duration_unit'];

	// Check if this pass is set to never expire
	if ( 'never' == $would_be_duration_unit ){
		$return_array = array(
			'end_time' => 'never',
			'would_expire' => false,
		);

		echo json_encode( $return_array );

		die();
	}

	$would_be_end_time = strtotime( $_POST['start_date'] . ' + ' . $would_be_duration_number . ' ' . $would_be_duration_unit, current_time( 'timestamp' ) );

	// If this would expire
	if( current_time( 'timestamp' ) > $would_be_end_time  ){
		$would_expire = true;
	}else{
		$would_expire = false;
	}

	$return_array = array(
		'end_time' => date( 'D, d M Y H:i:s', $would_be_end_time ),
		'would_expire' => $would_expire,
	);

	echo json_encode( $return_array );

	die();
}
add_action( 'wp_ajax_edd_all_access_expiration_check', 'edd_all_access_expiration_check' );
