<?php
/**
 * All Access Status Functions. This file contains all functions that *trigger* activating or deactivating an All Access pass.
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Check for any All Access Payments whose time period have passed and need to be expired.
 *
 * @since       1.0.0
 * @param       void
 * @return      void
 */
function edd_all_access_check_expired_periods(){

	//Get all Payments with All Access products listed as active
	$payments = new EDD_Payments_Query( array( 'meta_key' => '_edd_aa_active_ids' ) );
	$payments_with_active_aa = $payments->get_payments();
	$payments_to_check_for_expired_all_access = $payments_with_active_aa;

	// Loop through each given payment so we can check each one's expiry date.
	foreach( $payments_to_check_for_expired_all_access as $payment ){

		// Get the Payment Meta
		$payment_meta = $payment->get_meta();

		// Loop through all downloads purchased in this payment
		foreach( $payment_meta['downloads'] as $purchased_download ){

			if( isset( $purchased_download['options']['price_id'] ) ){
				$price_id = $purchased_download['options']['price_id'];
			}else{
				$price_id = 0;
			}

			$all_access_pass = new EDD_All_Access_Pass( $payment->ID, $purchased_download['id'], $price_id );

			// If it is required, expire this All Access Pass
			$all_access_pass->maybe_expire();

		}
	}
}

/**
 * When an All Access payment is freshly purchased, add it to the list of payments that need to be checked for expiration later.
 *
 * @since       1.0.0
 * @param       object $payment_id The ID payment being saved
 * @param       string $new_status
 * @param       string $old_status
 * @return      void
 */
function edd_all_access_check_updated_payment( $payment_id, $payment ){

	// If we were not passed an object
	if ( !is_object( $payment ) && is_int( $payment ) ){
		$payment = new EDD_Payment( $payment );
	}

	// Get the Payment Meta for the payment being saved at this moment.
	$payment_meta = $payment->get_meta();

	if ( !isset( $payment_meta['downloads'] ) ){
		return;
	}

	// Loop through all downloads purchased in this payment
	foreach( $payment_meta['downloads'] as $purchased_download ){

		$price_id = isset( $purchased_download['options']['price_id'] ) ? $purchased_download['options']['price_id'] : 0;

		$all_access_enabled = get_post_meta( $purchased_download['id'], '_edd_all_access_enabled', true ) ? true : false;

		// If the product being purchased/saved has All Access enabled, attempt to activate an All Access Pass for it.
		if ( $all_access_enabled ){

			$all_access_pass = new EDD_All_Access_Pass( $payment->ID, $purchased_download['id'], $price_id );

			// If it is required, activate this All Access Pass
			$all_access_pass->maybe_activate();
		}

	}

	// Check for any expired All Access Passes
	edd_all_access_check_expired_periods();

}
add_action( 'edd_payment_saved', 'edd_all_access_check_updated_payment', 10, 2 ); //Uncomment when edd 2.6.10 is released.

/**
 * When a payment status changes, check to see if any of it's purchased products should have All Access enabled or disabled.
 *
 * @since       1.0.0
 * @param       int $payment_id The ID of the payment being saved
 * @param       string $new_status The new status of the payment
 * @param       string $old_status The old status of the payment
 * @return      boolean/void
 */
function edd_all_access_check_updated_payment_on_status_change( $payment_id, $new_status, $old_status ){
	//Get the payment statuses that count as "valid" for All Access
	$valid_payment_statuses = apply_filters( 'edd_all_access_valid_statuses', array( 'publish', 'complete' ) );

	// All Access will neither activate or expire unless the status is valid.
	if ( ! in_array( $new_status, $valid_payment_statuses ) ){
		// Come back later when your status is valid (pending becomes complete sometimes after a Payment Gateway finalizes payments)
		return false;
	}

	edd_all_access_check_updated_payment( $payment_id, new EDD_Payment( $payment_id ) );
}
add_action( 'edd_update_payment_status', 'edd_all_access_check_updated_payment_on_status_change', 10, 3 );

/**
 * Once a day, check for any All Access Payments that may have expired.
 *
 * @since       1.0.0
 * @param       void
 * @return      void
 */
function edd_all_access_check_expired_periods_via_cron(){

	edd_all_access_check_expired_periods();

}
add_action( 'edd_daily_scheduled_events', 'edd_all_access_check_expired_periods_via_cron' );

/**
 * If edd_all_access_force_check_expirations is in the URL, do a check for expired payments.
 *
 * @since       1.0.0
 * @param       void
 * @return      void
 */
function edd_all_access_check_expired_periods_via_url(){

	if ( current_filter() == 'admin_init' && !isset( $_GET['edd_all_access_force_check_expirations'] ) ){
		return false;
	}

	edd_all_access_check_expired_periods();

}
add_action( 'admin_init', 'edd_all_access_check_expired_periods_via_url' );
