<?php
/**
 * This file contains all functions the handle/modify the way that the EDD Download Form is displayed to a user with All Access.
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Override the download form for users logged-in with an active All Access pass.
 * This is where the "Buy Now" button is turned into a "Download Now" button.
 *
 * @since       1.0.0
 * @param       string $form The existing download form
 * @param       array $args Arguments passed to the form
 * @return      string $form The updated download form
 */
function edd_all_access_download_form( $purchase_form, $args ) {

	global $post, $edd_displayed_form_ids;

	// Make sure the user is logged in before changing anything on the download form. For logged out users, nothing changes.
	if ( ! is_user_logged_in() ){
		return $purchase_form;
	}

	$post_id = is_object( $post ) ? $post->ID : 0;

	$download_id = absint( $args['download_id'] );
	$price_id = defined( 'DOING_AJAX' ) && DOING_AJAX ? absint( $args['price_id'] ) : edd_get_default_variable_price( $download_id );

	// This filter can be used by extensions to Ignore All Access for products that don't make sense - like products that aren't actually download-able (IE Bookings).
	$allow_all_access = apply_filters( 'edd_all_access_allow', true, $purchase_form, $args, $download_id, $price_id  );

	if ( ! $allow_all_access ){
		return $purchase_form;
	}

	$defaults = apply_filters( 'edd_purchase_link_defaults', array(
		'download_id' => $post_id,
		'price'       => (bool) false,
		'price_id'    => isset( $args['price_id'] ) ? $args['price_id'] : false,
		'style'       => edd_get_option( 'button_style', 'button' ),
		'color'       => edd_get_option( 'checkout_color', 'blue' ),
		'class'       => 'edd-submit'
	) );

	$args = wp_parse_args( $args, $defaults );

	// Check if this download is an "All Access" product. If it is, return the normal purchase form - you can't download an All Access product - you can only buy them.
	$all_access_enabled = get_post_meta( $download_id, '_edd_all_access_enabled', true );

	if ( $all_access_enabled ){
		return $purchase_form;
	}

	$all_access_check_args = array(
		'download_id' => $download_id,
		'price_id' => $price_id
	);

	$all_access = edd_all_access_check( $all_access_check_args );

	// If this user doesn't have access to the download+price_id in question
	if ( ! $all_access['success'] ){

		// If debugging, uncomment the line below and view the product you wish to download. It will tell you why this customer does not have all access to this product.
		//print_r($all_access); die();

		// Check if they have access to ANY of the variables price in this product
		$variable_prices = edd_get_variable_prices( $download_id );

		$can_access_at_least_one = false;

		foreach ( $variable_prices as $variable_price_id => $variable_price_data ){

			$variable_price_all_access_check_args = array(
				'download_id' => $download_id,
				'price_id' => $variable_price_id
			);

			$variable_price_all_access = edd_all_access_check( $variable_price_all_access_check_args );

			// If the customer does have access to at least one of the variable prices, set the $at_least_one variable to true
			if ( $variable_price_all_access['success'] ){
				$can_access_at_least_one = true;
			}

		}

		// If the customer cannot access at least one of the variable prices with an All Access pass, return the normal purchase form as is.
		if ( ! $can_access_at_least_one ){
			// Return the normal form
			return $purchase_form;
		}else{

			// If we got to here, we know that the customer does not have access to the default variable price id but DOES have access to at least one of the variable prices.
			// Now, lets check whether the site owner has chosen to hide non relevant price IDs.
			$hide_non_relevant_prices = edd_get_option( 'all_access_hide_non_relevant_variable_prices', 'no' );

			// If we do want to hide non relevant prices
			if ( 'yes' == $hide_non_relevant_prices ){

				//Reset the default price id chosen on the radio button to the first one that will be shown
				add_filter( 'edd_variable_default_price_id', 'edd_all_access_modify_default_price_id', 10, 2 );

			}else{

				// Return the normal form
				return $purchase_form;
			}
		}

	}

	// We'll hook in a filter here which may hide variable prices which the customer does not have access to
	add_filter( 'edd_purchase_variable_prices', 'edd_all_access_hide_non_relevant_prices', 10, 2 );

	//Figure out what the "Download Now" button should say
	if( isset( $args['all_access_download_now_text'] ) ){
		$download_label = $args['all_access_download_now_text'];
	}else{
		$download_label = edd_get_option( 'all_access_download_now_text', __( 'Download Now', 'edd-all-access' ) );
	}

	$download_class = implode( ' ', array( $args['style'], $args['color'], trim( $args['class'] ), 'edd-all-access-btn button' ) );
	$button_href = edd_all_access_product_download_url( $download_id, $price_id );

	// Collect any form IDs we've displayed already so we can avoid duplicate IDs
	if ( isset( $edd_displayed_form_ids[ $download_id ] ) ) {
		$edd_displayed_form_ids[ $download_id ]++;
	} else {
		$edd_displayed_form_ids[ $download_id ] = 1;
	}

	// Get the ID for the download form
	$form_id = ! empty( $args['form_id'] ) ? $args['form_id'] : 'edd_purchase_' . $download_id;

	// If we've already generated a form ID for this download ID, append -#
	if ( $edd_displayed_form_ids[ $download_id ] > 1 ) {
		$form_id .= '-' . $edd_displayed_form_ids[ $download_id ];
	}

	ob_start(); ?>

	<form id="<?php echo esc_attr( $form_id ); ?>" class="edd_download_purchase_form edd_all_access_download_form">

		<?php do_action( 'edd_purchase_link_top', $download_id, $args ); ?>

		<div class="edd_purchase_submit_wrapper">

		<?php echo sprintf(
			'<a class="%1$s" href="' . esc_url( $button_href ) . '"><span>%2$s</span></a>',
			$download_class,
			esc_attr( $download_label )
		); ?>

		</div>
	</form>

	<?php

	$form = ob_get_clean();

	// Remove the filters we added to this form
	remove_filter( 'edd_variable_default_price_id', 'edd_all_access_modify_default_price_id', 10, 2 );
	remove_filter( 'edd_purchase_variable_prices', 'edd_all_access_hide_non_relevant_prices', 10, 2 );

	return $form;
}
add_filter( 'edd_purchase_download_form', 'edd_all_access_download_form', 11, 2 );

/**
 * Add the CSS class "edd_all_access_price_options" to the price option HTML if this product can be downloaded using an All Access pass by this user.
 *
 * @since    1.0.0
 * @param    array $price_options_classes The array of all class names that will be output on the edd-price-options div
 * @param    int   $download_id The id of the product that is being output.
 * @return   array $price_options_classes The array of all class names that will be output on the edd-price-options div
 */
function edd_all_access_price_options_classes( $price_options_classes, $download_id ){

	$all_access = edd_all_access_check( array( 'download_id' => $download_id ) );

	if( ! $all_access['success'] ){
		return $price_options_classes;
	}

	// Add new, wanted classes
	$price_options_classes[] = 'edd_all_access_price_options';

	return $price_options_classes;

}
add_filter( 'edd_price_options_classes', 'edd_all_access_price_options_classes', 10, 2 );

/**
 * If this product can be downloaded using All Access, override the multi price mode to be OFF.
 * Multi Price mode needs to be off for All Access because you can only download 1 file at a time.
 * It won't zip multiple price options together for a single download. This likely will never become supported because tokenization wouldn't be supported.
 *
 * @since       1.0.0
 * @param       string $is_multi_enabled This will be "on" if true and empty if false
 * @param       int $download_id the ID of the download in question
 * @return      bool true if multi price mode should be enabled and false if not
 */
function edd_all_access_override_multi_price_mode( $is_multi_enabled, $download_id ){

	$all_access = edd_all_access_check( array( 'download_id' => $download_id ) );

	if( ! $all_access['success'] ){
		return $is_multi_enabled;
	}

	return false;

}
add_filter( 'edd_single_price_option_mode', 'edd_all_access_override_multi_price_mode', 10, 2 );

/**
 * When viewing a product with an All Access pass, if quantities are enabled, disable them here.
 * People don't need to download multiple copies of the same file at once.
 *
 * @since       1.0.0
 * @param       string $quantity_input The html for the quantities input field
 * @param       int $download_id The ID of the download in question
 * @args        array $args Arguements
 * @return      string $quantity_input an empty string since we do not want the quantities field
 */
function edd_all_access_disable_quantities( $quantity_input, $download_id, $args ) {

	$all_access = edd_all_access_check( array( 'download_id' => $download_id ) );

	if( ! $all_access['success'] ){
		return $quantity_input;
	}

	return NULL;

}
add_filter( 'edd_purchase_form_quantity_input', 'edd_all_access_disable_quantities', 10, 3 );

/**
 * Hide the price amount when showing variable prices to All Access customers.
 * For example, this will hide the "$1.00" on variable prices.
 *
 * @since   1.0.3
 * @param	string	$price_output	The HTML output of the variable price
 * @param	int		$download_id	The ID of the download being viewed
 * @param	int		$key			The key of this variable price in the array of variable prices for this product.
 * @param	array	$price			The array of data about this price
 * @param	string	$form_id		The HTML ID of the form containing these variable prices
 * @param	string	$item_prop		The HTML item prop attribute
 * @param	string	$price_output	The filtered/modified HTML output of the variable price
 */
function edd_all_access_remove_price_amounts( $price_output, $download_id, $price_id, $price, $form_id, $item_prop ){

	// Check if the customer has All Access to this price ID. If they do, don't show the price.
	$all_access_check_args = array(
		'download_id' => $download_id,
		'price_id' => $price_id
	);

	$all_access = edd_all_access_check( $all_access_check_args );

	// If this user does have access to this variable price
	if ( $all_access['success'] ){

		// Remove the price amount output
		$price_output = '<span class="edd_price_option_name"' . $item_prop . '>' . esc_html( $price['name'] ) . '</span>';

		return $price_output;
	}

	// Otherwise, return the price output as-is.
	return $price_output;

}
add_filter( 'edd_price_option_output', 'edd_all_access_remove_price_amounts', 10, 6 );

/**
* Should we hide non relevant variable prices from customers with All Access?
* For example, this is useful if you want to hide "Small" and "Medium" price options from customers who have access to the "Large" version.
*
* @since       1.0.2
* @param       array $variable_prices The array of variable prices that would be shown in the download form.
* @param       array $download_id The ID of the product being viewed.
* @return      array $variable_prices The modified array of variable prices that will be shown in the download form.
*/
function edd_all_access_hide_non_relevant_prices( $variable_prices, $download_id ){

   $hide_non_relevant_prices = edd_get_option( 'all_access_hide_non_relevant_variable_prices', 'no' );

   // If we shouldn't hide the non-relevant variable prices, return them as-is.
   if( 'no' == $hide_non_relevant_prices ){
	   return $variable_prices;
   }

   // Figure out which variable prices the customer has access to - and which ones they do not.
   foreach( $variable_prices as $variable_price_id => $variable_price_info ){

	   $all_access_check_args = array(
		   'download_id' => $download_id,
		   'price_id' => $variable_price_id
	   );

	   $all_access = edd_all_access_check( $all_access_check_args );

	   // If this user doesn't have access to this variable price, remove it from this list to show
	   if ( ! $all_access['success'] ){
		   unset( $variable_prices[$variable_price_id] );
	   }
   }

   return $variable_prices;

}

/**
 * Modify the default price ID to be the first one that the customer has All Access to. This is in a scenario where the first variable price is not included in All Access.
 *
 * @since  1.0.3
 * @param  int $price_id ID The default Price ID to select
 * @param  int $download ID The ID of the download being viewed or in question.
 * @return int $price_id ID The default Price ID to select
 */
function edd_all_access_modify_default_price_id( $price_id, $download_id ){

	// Get the variable prices
	$variable_prices = edd_get_variable_prices( $download_id );

	// Loop through them
	foreach ( $variable_prices as $variable_price_id => $variable_price_data ){

		$all_access_check_args = array(
			'download_id' => $download_id,
			'price_id' => $variable_price_id
		);

		$all_access = edd_all_access_check( $all_access_check_args );

		// If the customer does have access to at least one of the variable prices, stop looping and set the default variable price to that one.
		if ( $all_access['success'] ){
			return $variable_price_id;
		}
	}

	return $price_id;
}
