<?php
/**
 * All Access Download Functions. This file contains all functions that relate to *actually actually downloading* a file using an All Access pass.
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ){
	exit;
}

/**
* Used to deliver a download to a user without revealing the actual URL.
*
* @since    1.0.0
* @return   void
*/
function edd_all_access_convert_site_to_download(){

	// If our "download" variable is set in the URL
	if ( !isset( $_GET['edd-all-access-download'] )){
		return false;
	}

	// Get the download ID we will deliver
	$download_to_deliver = is_numeric( $_GET['edd-all-access-download'] ) ? $_GET['edd-all-access-download'] : NULL;
	$price_id_to_deliver = isset( $_GET['edd-all-access-price-id'] ) && is_numeric( $_GET['edd-all-access-price-id'] ) ? $_GET['edd-all-access-price-id'] : false;

	$args = array(
		'download_id' => $download_to_deliver,
		'price_id' => $price_id_to_deliver,
		'check_download_limit' => true //Make sure to check for download limits here since we are actually attempting to download a file.
	);

	// Check if this user has an All Access priveleges for this product.
	$all_access_check = edd_all_access_check( $args );
	$all_access_pass = $all_access_check['all_access_pass'];

	// If the user does NOT have All Access priveleges for this product
	if ( ! $all_access_check['success'] ){

		if ( 'all_access_pass_expired' == $all_access_check['failure_id'] ){
			$redirect_url = edd_get_option( 'all_access_expired_redirect' );

			// If a redirect url has been set up for expired passes, redirect them there. This is useful for custom pages with a message on how to renew.
			if ( ! empty( $redirect_url ) ){
				wp_redirect( esc_url( $redirect_url ) );
				exit();
			}

		}

		if ( 'category_not_included' == $all_access_check['failure_id'] ){
			$redirect_url = edd_get_option( 'all_access_category_not_included_redirect' );

			// If a redirect url has been set up for category-not-included failure, redirect them there.
			if ( ! empty( $redirect_url ) ){
				wp_redirect( esc_url( $redirect_url ) );
				exit();
			}

		}

		if ( 'price_id_not_included' == $all_access_check['failure_id'] ){
			$redirect_url = edd_get_option( 'all_access_price_id_not_included_redirect' );

			// If a redirect url has been set up for category-not-included failure, redirect them there.
			if ( ! empty( $redirect_url ) ){
				wp_redirect( esc_url( $redirect_url ) );
				exit();
			}

		}

		if ( 'download_limit_reached' == $all_access_check['failure_id'] ){

			$error_message = edd_get_option( 'all_access_download_limit_reached_text', __( 'Sorry. You\'ve hit the maximum number of downloads allowed for your All Access account.', 'edd-all-access' ) ) . ' (' . edd_all_access_download_limit_string( $all_access_pass )  . ')';

			$redirect_url = edd_get_option( 'all_access_download_limit_reached_redirect' );

			// If a redirect url has been set up for category-not-included failure, redirect them there.
			if ( ! empty( $redirect_url ) ){

				wp_redirect( esc_url( $redirect_url ) );
				exit();
			}else{

				// Download Limit Reached Error page
				wp_die( apply_filters( 'edd_all_access_download_limit_reached_message', $error_message ), __( 'Error', 'edd-all-access' ), array( 'response' => 403 ) );

				exit();
			}

		}

		// Default error page if none others have prompted a separate action
		wp_die( apply_filters( 'edd_all_access_no_all_access_message', $all_access_check['failure_message'], $all_access_check['failure_id'] ), __( 'Error', 'edd-all-access' ), array( 'response' => 403 ) );

		exit();
	}

	// Get the user ID
	$user_id = get_current_user_id();

	// Get the customer
	$customer = new EDD_Customer( $user_id, true );

	// If the currently logged-in customer is not the one attached to this All Access Pass
	if ( $customer->id != $all_access_pass->customer->id ){

		wp_die( apply_filters( 'edd_all_access_no_all_access_message', __( 'That All Access Pass does not belong to this account. Try logging in with the correct account information', 'edd-all-access' ), 'wrong_account' ), __( 'Error', 'edd-all-access' ), array( 'response' => 403 ) );

		exit();
	}

	// If the download ID didn't pass sanitization
	if ( !$download_to_deliver ){
		$error_message = __( 'Download ID sanitization failed.', 'edd-all-access' );

		wp_die( apply_filters( 'edd_all_access_sanitization_fail_message', $error_message ), __( 'Error', 'edd-all-access' ), array( 'response' => 403 ) );

		exit();
	}

	$hours = absint( edd_get_option( 'download_link_expiration', 24 ) );

	if ( ! ( $date = strtotime( '+' . $hours . 'hours', current_time( 'timestamp') ) ) ) {
		$date = 2147472000; // Highest possible date, January 19, 2038
	}

	// Get the array of deliverable files so we can get the array key for the file we want
	$files = edd_get_download_files( $download_to_deliver, $price_id_to_deliver );

	foreach( $files as $file_key => $file_url ){
		$file_key = $file_key;
	}

	// Don't deliver if this is no file attached to that Price ID
	if( !isset( $file_key ) || $price_id_to_deliver != 0 && !$file_key ){
		$error_message = __( 'Oops! There is no file attached to that price. Please contact the site administrator.', 'edd-all-access' ) . '<br />' . __( 'Download ID:', 'edd-all-access' ) . ' ' . $download_to_deliver . ' | ' . __( 'Price ID:', 'edd-all-access' ) . ' ' . $price_id_to_deliver;

		wp_die( apply_filters( 'edd_all_access_no_file_key_message', $error_message ), __( 'Error', 'edd-all-access' ), array( 'response' => 403 ) );

		exit();
	}

	// Reset/Hotwire some of the variables used to process the download.
	$_GET['download_id']	= $download_to_deliver;
	$_GET['expire']			= $date;
	$_GET['file']			= $file_key;
	$_GET['price_id']		= $price_id_to_deliver;
	$_GET['key']			= NULL;
	$_GET['eddfile']		= sprintf( '%d:%d:%d:%d', $all_access_pass->payment_id, $download_to_deliver, $file_key, $price_id_to_deliver );
	$_GET['ttl']			= rawurlencode( $date );
	$_GET['all_access_pass_id'] = $all_access_pass->id;

	// Make sure the user will have access
	add_filter( 'edd_file_download_has_access', '__return_true' );
	add_filter( 'edd_process_download_args', 'edd_all_access_add_payment_to_download_args' );
	add_filter( 'edd_url_token_allowed_params', 'edd_all_access_url_tokens_add_params' );

	// Note that we increment the count of downloads-used by the customer in the edd_process_download_headers hook

	// Now that we've authenticated the All Access, process the download
	edd_process_download();

}
add_action('init', 'edd_all_access_convert_site_to_download' );

/**
 * After a download is completed by a customer using an All Access pass, add some meta to the log letting it know which All Access download was used.
 *
 * @since    1.0.0
 * @return   array $download_args The args being set in the edd_process_download function.
 */
function edd_all_access_add_download_id_to_file_log( $requested_file, $download, $email, $payment ){

	// If no All Access Pass was used to download this file, get out of here now.
	if( ! isset( $_GET['all_access_pass_id'] )|| empty( $_GET['all_access_pass_id'] ) ){
		return false;
	}

	// Lets set up the ALl Access Pass object using the id from the url
	$aa_data = explode( '_', $_GET['all_access_pass_id'] );
	$payment_id = intval( $aa_data[0] );
	$download_id = intval( $aa_data[1] );
	$price_id = intval( $aa_data[2] );
	$all_access_pass = new EDD_All_Access_Pass( $payment_id, $download_id, $price_id );

	// If this All Access Pass is not valid
	if ( 'active' != $all_access_pass->status ){
		return false;
	}

	// Get the ID of our newly added download log
	$log_id = get_posts("post_type=edd_log&numberposts=1&fields=ids");

	// Since the purchased All Access ID will be different than the ID of the downloaded post, we'll also add that All Access ID to the log.
	update_post_meta( $log_id[0], '_edd_log_all_access_pass_id', sanitize_text_field( $all_access_pass->id ) );

	// Increment the number of downloads-used by this customer in this time period
	$all_access_pass->downloads_used = $all_access_pass->downloads_used + 1;
}
add_action( 'edd_process_download_headers', 'edd_all_access_add_download_id_to_file_log', 10, 4 );

/**
 * When a customer is downloading a product, add the payment ID containing this All Access pass to the download args.
 * This is hooked to edd_process_download_args when a download using All Access is actually taking place.
 *
 * @since    1.0.0
 * @param    array $download_args The args being set in the edd_process_download function.
 * @return   array $download_args The args being set in the edd_process_download function.
 */
function edd_all_access_add_payment_to_download_args( $download_args ){

	$edd_all_access_check = edd_all_access_check( array(
		'download_id' => $download_args['download'],
		'price_id' => $download_args['price_id'],
	) );

	// If the all access check failed, dont add anything to the download log args
	if ( ! $edd_all_access_check['success'] ){
		return $download_args;
	}

	$download_args['payment'] = $edd_all_access_check['all_access_pass']->payment_id;

	$customer = new EDD_Customer( get_current_user_id(), true );
	$download_args['email'] = $customer->email;

	return $download_args;
}

/**
 * Set the allowed token paramaters to include the edd-all-access-download URL paramater
 * This is hooked to edd_url_token_allowed_params when an All Access download is taking place.
 *
 * @since    1.0.0
 * @param    array $token_params The args being set in the edd_process_download function.
 * @return   array $token_params The args being set in the edd_process_download function.
 */
function edd_all_access_url_tokens_add_params( $token_params ){
	$token_params[] = 'edd-all-access-download';
	$token_params[] = 'edd-all-access-price-id';
	return $token_params;
}
