<?php
/**
 * Utility/Helper Functions
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Check if a user has access to a specific EDD Download through an "All Access" purchase.
 * All we need to know for this function is which product/price-variation is being downloaded.
 * From there we can detect the user based on whether they are logged in.
 *
 * Note: in a case where a user has purchased multiple All Access passes, the first one that qualifies will be returned.
 *
 * @since    1.0.0
 * @param    array $args Array of options that determine the return
 * @return   array - Array of success or failure messages depending on access.
 */
function edd_all_access_check( $args = array() ){

	$default_args = array(
		'download_id' => false,
		'price_id' => false,
		'customer_id' => false,
		'user_id' => false,
		'check_download_limit' => false, // by default we don't check the download limit because this function is used in places other than just downloading.
		'require_login' => true
	);

	$args = wp_parse_args( $args, $default_args );

	//Set up the default return data
	$return_data = array(
		'success' => false,
		'all_access_pass' => false,
		'failure_id' => 'failure_by_default',
		'failure_message' => __( 'For some unknown reason, this user does not have All Access to this product.', 'edd-all-access' ),
	);

	// First, if required, lets make sure this user is even logged in
	if ( $args['require_login'] && ( ! is_user_logged_in() || 0 == get_current_user_id() ) ){

		$return_data = array(
			'success' => false,
			'all_access_pass' => false,
			'failure_id' => 'user_not_logged_in',
			'failure_message' => __( 'You must be logged in to have access to this file.', 'edd-all-access' )
		);

		return $return_data;
	}

	// If this product is known to be excluded from all All Access passes.
	$product_is_excluded = get_post_meta( $args['download_id'], '_edd_all_access_exclude', true );

	if ( $product_is_excluded ){
		$return_data = array(
			'success' => false,
			'all_access_pass' => false,
			'failure_id' => 'product_is_excluded',
			'failure_message' => __( 'The product you are attempting to access is excluded from All Access', 'edd-all-access' ),
		);
		return $return_data;
	}

	// If no edd customer is given
	if( ! $args['customer_id'] ) {
		// If no user id was provided, use the current user
		if( ! $args['user_id'] ) {
			$person_id = get_current_user_id();
			$by_user_id = true;
		}else{
			$person_id = $args['user_id'];
			$by_user_id = true;
		}
	}else{
		// If a customer was provided
		$person_id = $args['customer_id'];
		$by_user_id = false;
	}

	$customer = new EDD_Customer( $person_id, $by_user_id );

	// Get the All Access passes saved to this customer meta
	$customer_all_access_passes = $customer->get_meta( 'all_access_passes' );

	// If this customer has no all access passes, they don't have access for sure.
	if ( empty( $customer_all_access_passes ) ){

		$return_data = array(
			'success' => false,
			'all_access_pass' => NULL,
			'failure_id' => 'no_all_access_passes_purchased',
			'failure_message' => __( 'You have no purchased any All Access Passes', 'edd-all-access' )
		);

		return $return_data;
	}

	// Loop through all of this customer's All Access Passes
	foreach( $customer_all_access_passes as $purchased_download_id_price_id => $purchased_aa_data ){

		// In case there happens to be an entry in the array without a numeric key
		if ( empty( $purchased_download_id_price_id ) ){
			continue;
		}

		if ( ! isset( $purchased_aa_data['payment_id'] ) || ! isset( $purchased_aa_data['download_id'] ) || ! isset( $purchased_aa_data['price_id'] ) ){
			continue;
		}

		// Set up the All Access Pass object
		$all_access_pass = EDD_All_Access_Pass( $purchased_aa_data['payment_id'], $purchased_aa_data['download_id'], $purchased_aa_data['price_id'] );

		// If this All Access Pass is not valid for use
		if ( 'invalid' == $all_access_pass->status || 'upcoming' == $all_access_pass->status || is_wp_error( $all_access_pass->status )  ){
			// Skip this item and check the next item that was in the customer's cart for this payment.
			continue;
		}

		// Check the status of this All Access Payment. If it is expired, skip it.
		if ( 'expired' == $all_access_pass->status ){

			// Run the expiration method to make sure everything is properly set for this expired All Access Pass
			$all_access_pass->maybe_expire();

			// If a previous purchased product gave us all access but this one does not, retain the previous value.
			// In other words, only fail this if a previous result was not success. Previous success remains.
			// If a previous failure exists and came from a valid, non-expired All Access pass, use that failure instead of this one.
			// Only overwrite a previous failure with this expired pass's data if the failure is the default.
			if ( ! $return_data['success'] && $return_data['failure_id'] == 'failure_by_default'){

				$expired_text = edd_get_option( 'all_access_expired_text', __( 'Your All Access Pass is expired.', 'edd-all-access' ) );

				$return_data = array(
					'success' => false,
					'all_access_pass' => $all_access_pass,
					'failure_id' => 'all_access_pass_expired',
					'failure_message' => $expired_text
				);
			}

			// Skip further checks using settings from this expired All Access pass.
			continue;
		}

		// Check the status of this All Access Payment. If it is upgraded, skip it.
		if ( 'upgraded' == $all_access_pass->status ){

			// If a previous purchased product gave us all access but this one does not, retain the previous value.
			// In other words, only fail this if a previous result was not success. Previous success remains.
			// If a previous failure exists and came from a valid, non-expired All Access pass, use that failure instead of this one.
			// Only overwrite a previous failure with this expired pass's data if the failure is the default.
			if ( ! $return_data['success'] && $return_data['failure_id'] == 'failure_by_default'){

				// In theory this will never actually be shown because the upgraded-to all access pass takes over.
				$upgraded_text = edd_get_option( 'all_access_upgraded_text', __( 'This All Access Pass was upgraded to another one.', 'edd-all-access' ) );

				$return_data = array(
					'success' => false,
					'all_access_pass' => $all_access_pass,
					'failure_id' => 'all_access_pass_upgraded',
					'failure_message' => $upgraded_text
				);
			}

			// Skip further checks using settings from this expired All Access pass.
			continue;
		}

		// Check the status of this All Access Payment. If it is renewed, skip it.
		if ( 'renewed' == $all_access_pass->status ){

			// If a previous purchased product gave us all access but this one does not, retain the previous value.
			// In other words, only fail this if a previous result was not success. Previous success remains.
			// If a previous failure exists and came from a valid, non-expired All Access pass, use that failure instead of this one.
			// Only overwrite a previous failure with this expired pass's data if the failure is the default.
			if ( ! $return_data['success'] && $return_data['failure_id'] == 'failure_by_default'){

				// In theory this will never actually be shown because the renewed-to all access pass takes over.
				$upgraded_text = edd_get_option( 'all_access_renewed_text', __( 'This All Access Pass was renewed. It is no longer active but the newest one is.', 'edd-all-access' ) );

				$return_data = array(
					'success' => false,
					'all_access_pass' => $all_access_pass,
					'failure_id' => 'all_access_pass_renewed',
					'failure_message' => $upgraded_text
				);
			}

			// Skip further checks using settings from this expired All Access pass.
			continue;
		}

		// Make sure the All Access Pass status is "active"
		if ( $all_access_pass->status != 'active' ){
			// If a previous purchased product gave us all access but this one does not, retain the previous value.
			// In other words, only fail this if a previous result was not success. Previous success remains.
			// If a previous failure exists and came from a valid, non-expired All Access pass, use that failure instead of this one.
			// Only overwrite a previous failure with this expired pass's data if the failure is the default.
			if ( ! $return_data['success'] && $return_data['failure_id'] == 'failure_by_default'){
				$return_data = array(
					'success' => false,
					'all_access_pass' => $all_access_pass,
					'failure_id' => 'all_access_pass_not_active',
					'failure_message' => __( 'Your All Access Pass is not active.', 'edd-all-access' )
				);
			}

			// Skip further checks using settings from this expired All Access pass.
			continue;
		}

		// Get the product-being-downloaded's information
		$download_categories = wp_get_post_terms( $args['download_id'], 'download_category' );
		$has_required_category = false;

		// If we should include a check against the download limit (by default we do not)
		if ( $args['check_download_limit'] ){

			// If the downloads used are greater than the download limit
			if ( $all_access_pass->downloads_used >= $all_access_pass->download_limit && $all_access_pass->download_limit != 0 ){
				$return_data = array(
					'success' => false,
					'all_access_pass' => $all_access_pass,
					'failure_id' => 'download_limit_reached',
					'failure_message' => edd_get_option( 'all_access_download_limit_reached_text', __( 'Sorry. You\'ve hit the maximum number of downloads allowed for your All Access account.', 'edd-all-access' ) ) . ' (' . edd_all_access_download_limit_string( $all_access_pass )  . ')'
				);

				// Skip further checks using settings from this download-limit-reached All Access pass.
				continue;
			}

		}

		// Loop through each acceptable/included download category
		foreach( $all_access_pass->included_categories as $included_category ){

			//Loop through each category attached to the download the user wishes to download/access
			foreach( $download_categories as $download_category ){
				if ( $included_category == $download_category->term_id ){
					$has_required_category = true;
					break;
				}
			}
		}

		// If this purchase gives All Access to a category that this download has:
		if ( in_array( 'all', $all_access_pass->included_categories ) || $has_required_category ){

			// If this purchase gives All Access to the price_id we are hoping to retrieve
			if ( empty( $args['price_id'] ) || ! $all_access_pass->included_price_ids || in_array( $args['price_id'], $all_access_pass->included_price_ids ) ){

				// Get all the variable price settings saved to the product's variable prices.
				$variable_prices = edd_get_variable_prices( $args['download_id'] );

				// If this price id is excluded from all access within the actual post's settings
				if ( ! empty( $args['price_id'] ) && isset( $variable_prices[ $args['price_id'] ] ) && ! empty( $variable_prices[ $args['price_id'] ][ 'excluded_price' ] ) ) {

					// If a previous purchased product gave us all access but this one does not, retain the previous value.
					// In other words, only fail this if a previous result was not success. Previous success remains.
					// If a previous failure exists and came from a valid, non-expired All Access pass, use that failure instead of this one.
					// Only overwrite a previous failure with this expired pass's data if the failure is the default.
					if ( ! $return_data['success'] && $return_data['failure_id'] == 'failure_by_default'){

						$price_id_not_included_text = edd_get_option( 'all_access_price_id_not_included_text', __( 'Your account does not have access to this product variation.', 'edd-all-access' ) );

						$return_data = array(
							'success' => false,
							'all_access_pass' => $all_access_pass,
							'failure_id' => 'price_id_not_included',
							'failure_message' => $price_id_not_included_text
						);
					}

				}else{

					// If this price id is not excluded from All Access in any way
					$return_data = array(
						'success' => true,
						'all_access_pass' => $all_access_pass,
						'failure_id' => 'no_failure',
						'failure_message' => ''
					);

					// Return as soon as we find a successful All Access pass for this product. No further checking is required.
					return $return_data;

				}

			}
			else{

				// If a previous purchased product in this payment gave us all access but this one does not, retain the previous value.
				// In other words, only fail this if a previous result was not success. Previous success remains.
				if ( ! $return_data['success'] ){

					$price_id_not_included_text = edd_get_option( 'all_access_price_id_not_included_text', __( 'Your account does not have access to this product variation.', 'edd-all-access' ) );

					$return_data = array(
						'success' => false,
						'all_access_pass' => $all_access_pass,
						'failure_id' => 'price_id_not_included',
						'failure_message' => $price_id_not_included_text
					);

				}
			}
		}
		else{
			// If a previous purchased product in this payment gave us all access but this one does not, retain the previous value.
			// In other words, only fail this if a previous result was not success. Previous success remains.
			if ( ! $return_data['success'] ){

				$cat_not_included_text = edd_get_option( 'all_access_category_not_included_text', __( 'Your account does not have access to products in this category.', 'edd-all-access' ) );

				$return_data = array(
					'success' => false,
					'all_access_pass' => $all_access_pass,
					'failure_id' => 'category_not_included',
					'failure_message' => $cat_not_included_text
				);
			}
		}
	}

	return $return_data;

}

/**
* Get an array of product/download ids which are "All Access" enabled.
*
* @since    1.0.0
* @link     http://mintplugins.com/doc/
* @param    bool $refresh - Whether or not to search the database for Downloads with All Access enabled or just use the existing values saved to the database option.
* @return   array - The post ids of all "All Access" posts
*/
function edd_all_access_get_all_access_downloads(){

	// Cache this for any subsequent calls on this page
	global $edd_all_access_downloads;

	if ( ! empty( $edd_all_access_downloads ) ){
		return $edd_all_access_downloads;
	}

	$edd_all_access_products = array();

	// Check if a cache clear needs to happen for hosts with overly aggressive object caching.
	if( file_exists( WP_CONTENT_DIR . '/object-cache.php' ) ){
	  wp_cache_flush();
	}

	//If we should just use the values saved to the database.
	$edd_all_access_products = get_option( 'edd_all_access_products' );

	// Double check each post listed to make sure it exists and has All Access enabled
	$double_checked_edd_all_access_products = array();

	if ( ! empty( $edd_all_access_products ) ){
		foreach( $edd_all_access_products as $all_access_product ){

			// If this post exists
			$product = new EDD_Download( $all_access_product );

			// If this post exists and is an EDD download
			if( 0 != $product->ID ){

				// If it is published
				if( 'publish' == $product->post_status ){

					$all_access_enabled = get_post_meta( $all_access_product, '_edd_all_access_enabled', true );

					// If All Access is enabled
					if( ! empty( $all_access_enabled ) ){

						// Add it back to the list as it has been officially double checked
						$double_checked_edd_all_access_products[] = $all_access_product;

					}

				}
			}
		}
	}

	$double_checked_edd_all_access_products = is_array( $double_checked_edd_all_access_products ) ? array_unique( $double_checked_edd_all_access_products ) : array();

	// If after double checking the list of All Access products, they are different, update the option.
	if ( $double_checked_edd_all_access_products != $edd_all_access_products ){
		update_option( 'edd_all_access_products', $double_checked_edd_all_access_products );
	}

	return apply_filters( 'edd_all_access_downloads', $double_checked_edd_all_access_products );

}

/**
* Check if a download is an All Access enabled product.
*
* @since    1.0.0
* @param    int $download_id The id of the product being checked
* @param    int $price_id The price id of the product being checked.
* @return   int $download_id The id of the product being checked
*/
function edd_all_access_download_is_all_access( $download_id, $price_id = 0 ){

   // If this purchase is not an "All Access" post
   if ( ! in_array( $download_id, edd_all_access_get_all_access_downloads() ) ){
	   return false;
   }

   return true;

}

/**
 * Check if an All Access pass (if theoretically purchased and active) would have access to an EDD Download. This is not dependant on any user or payment.
 *
 * @since    1.0.0
 * @param    array $args Array of options that determine the return
 * @return   array This array contains either a success or failure message why it failed.
 */
function edd_all_access_includes_download( $args = array() ){

	$default_args = array(
		'all_access_product_info' => array(
			'download_id' => 0,
			'price_id' => 0 // This is unused even if variable pricing is enabled for an All Access Pass because the meta options are product-wide (not pricing specific).
		),
		'desired_product_info' => array(
			'download_id' => 0,
			'price_id' => 0,
		)
	);

	$args = wp_parse_args( $args, $default_args );

	// Get all posts which count as "All Access" posts.
	$all_access_posts = edd_all_access_get_all_access_downloads();

	// There's no All Access to purchase so definitively, none could access this product.
	if ( empty($all_access_posts) ){
		return array(
			'success' => false,
			'failure_id' => 'no_all_access_posts_exist',
			'failure_message' => __( 'No All Access downloads have been configured', 'edd-all-access' )
		);
	}

	// If the passed in download_id is not an All Access pass
	if ( ! in_array( $args['all_access_product_info']['download_id'], $all_access_posts ) ){
		return array(
			'success' => false,
			'failure_id' => 'not_an_all_access_product',
			'failure_message' => __( 'The ID passed was not an All Access product.', 'edd-all-access' )
		);
	}

	// Check if this product being viewed has variable pricing
	$has_variable_prices = edd_has_variable_prices( $args['desired_product_info']['download_id'] );

	// If there are variable prices and the desired product is one of those variable prices (not single price mode)
	if ( $has_variable_prices && $args['desired_product_info']['price_id'] != 0 ) {

		// Check if any of the variable prices exclude All Access.
		$variable_prices = edd_get_variable_prices( $args['desired_product_info']['download_id'] );

		foreach( $variable_prices as $variable_price_id => $variable_price_settings ){

			// If this variable price id matches the one we are checking for with this function
			if ( $variable_price_id == $args['desired_product_info']['price_id'] ){

				// If this variable price is excluded from All Access
				if( ! empty( $variable_price_settings[ 'excluded_price' ] ) ){
					$return_data = array(
						'success' => false,
						'failure_id' => 'product_is_excluded',
						'failure_message' => __( 'The Price ID you are attempting to access is excluded from All Access', 'edd-all-access' ),
					);
					return $return_data;
				}
			}
		}
	}
	else{
		// If this single-price product is known to be excluded from all All Access passes.
		$product_is_excluded = get_post_meta( $args['desired_product_info']['download_id'], '_edd_all_access_exclude', true );

		if ( $product_is_excluded ){
			$return_data = array(
				'success' => false,
				'failure_id' => 'product_is_excluded',
				'failure_message' => __( 'The product you are attempting to access is excluded from All Access', 'edd-all-access' ),
			);
			return $return_data;
		}
	}

	// Get the all access meta information
	$meta = get_post_meta( $args['all_access_product_info']['download_id'], '_edd_all_access_settings', true );
	$all_access_categories = isset( $meta['all_access_categories']  ) ? $meta['all_access_categories']  : array( 'all' );
	$included_price_ids = isset( $meta['all_access_included_price_ids']  ) ? $meta['all_access_included_price_ids']  : 0;

	//Get the Categories assigned to the desired download.
	$download_categories = wp_get_post_terms( $args['desired_product_info']['download_id'], 'download_category' );
	$has_required_category = false;

	if ( $all_access_categories != 'all' ){
		// Loop through each acceptable/included download category
		foreach( $all_access_categories as $all_access_category ){

			//Loop through each category attached to the download the user wishes to download/access
			foreach( $download_categories as $download_category ){
				if ( $all_access_category == $download_category->term_id ){
					$has_required_category = true;
					break;
				}
			}
		}
	}

	// If this All Access Product has access to a category that this download has:
	if ( empty( $all_access_categories ) || in_array( 'all', $all_access_categories ) || $has_required_category ){

		// If this All Access Product has access to the price_id we are hoping to retrieve
		if ( empty( $args['desired_product_info']['price_id'] ) || empty( $included_price_ids ) || in_array( $args['desired_product_info']['price_id'], $included_price_ids ) ){

			// The All Access post would/does include access to the Download/Price ID requested
			return array(
				'success' => true,
				'success_message' => __( 'This All Access post does include access to this Download.', 'edd-all-access' ),
				'failure' => false,
			);
		}
		else{
			return array(
				'success' => false,
				'failure' => 'price_id_not_included',
				'failure_message' => __( 'This All Access post does not have access to this product variation.', 'edd-all-access' )
			);
		}
	}
	else{
		return array(
			'success' => false,
			'failure' => 'category_not_included',
			'failure_message' => __( 'This All Access post does not have access to products in this category.', 'edd-all-access' )
		);
	}

	return array(
		'success' => false,
		'failure' => 'unknown_error',
		'failure_message' => __( 'Unknown error with All Access', 'edd-all-access' )
	);
}

/**
 * Get the URL used to download products using an All Access pass.
 *
 * @since    1.0.0
 * @param    int - The post ID of the product being downloaded
 * @param    int - The price ID product being downloaded
 * @return   string - The URL used to downlodad the file via All Access
 */
function edd_all_access_product_download_url( $download_id, $price_id = 0 ){

	if ( empty( $price_id ) ){
		$price_id = edd_get_default_variable_price( $download_id );
	}

	if ( empty( $price_id ) ){
		$button_href = add_query_arg( array(
			'edd-all-access-download' => $download_id,
		),  get_bloginfo( 'wpurl' ) );
	}else{
		$button_href = add_query_arg( array(
			'edd-all-access-download' => $download_id,
			'edd-all-access-price-id' => $price_id
		),  get_bloginfo( 'wpurl' ) );
	}

	$download_token = edd_get_download_token( $button_href );

	return esc_url( add_query_arg( array( 'token' => $download_token ), $button_href ) );
}

/**
 * Get the status label of an All Access pass.
 *
 * @since    1.0.0
 * @param    string - The status of the All Access Pass.
 * @return   string - "Active" if still active. 'Expired' if expired. 'Invalid' if invalid.
 */
function edd_all_access_get_status_label( $status ){
	switch( $status ) {
		case 'active' :
			$status = __( 'Active', 'edd-all-access' );
			break;

		case 'expired' :
			$status = __( 'Expired', 'edd-all-access' );
			break;

		case 'upgraded' :
				$status = __( 'Upgraded', 'edd-all-access' );
				break;

		case 'renewed' :
				$status = __( 'Renewed', 'edd-all-access' );
				break;

		case 'upcoming' :
				$status = __( 'Upcoming', 'edd-all-access' );
				break;

		case 'invalid' :
			$status = __( 'Invalid', 'edd-all-access' );
			break;
	}

	return $status;
}

/**
 * Get the PHP acceptable time string for the download limit time period (Downloads per X).
 *
 * @since    1.0.0
 * @param    string - The saved meta for the download limit time period.
 * @return   string - The PHP str_to_time acceptable word equivalent.
 */
function edd_all_access_download_limit_time_period_to_string( $download_limit_time_period, $allow_translations = true ){
	// If the time period is a day - get the string 'day' to show the user
	if ( $download_limit_time_period == 'per_day' ){
		$time_period = $allow_translations ? __( 'day', 'edd-all-access' ) : 'day';
	}
	// If the time period is a week - get the string 'week' to show the user
	else if ( $download_limit_time_period == 'per_week' ){
		$time_period = $allow_translations ? __( 'week', 'edd-all-access' ) : 'week';
	}
	// If the time period is a month - get the string 'month' to show the user
	else if ( $download_limit_time_period == 'per_month' ){
		$time_period = $allow_translations ? __( 'month', 'edd-all-access' ) : 'month';
	}
	// If the time period is a year - get the string 'year' to show the user
	else if ( $download_limit_time_period == 'per_year' ){
		$time_period = $allow_translations ? __( 'year', 'edd-all-access' ) : 'year';
	}
	// If the time period is a per period - get the string 'total' to show the user
	else if ( $download_limit_time_period == 'per_period' ){
		$time_period = $allow_translations ? __( 'total', 'edd-all-access' ) : 'total';
	}

	return $time_period;
}

/**
 * Turn $download_limit per $time_period into a human-readable string like "5 Downloads per Day".
 *
 * @since    1.0.0
 * @param    object - An All Access Pass object
 * @return   string - Easy to read string representing the download limit for an All Access Pass.
 */
function edd_all_access_download_limit_string( $all_access_pass ){

	if( 0 == $all_access_pass->download_limit ){
		$assembled_string = __( 'Unlimited downloads', 'edd-all-access' );
	}else{
		$per_string = _x( 'per', 'Translating "per" in "5 Downloads per day"','edd-all-access' );

		$time_period_string = edd_all_access_download_limit_time_period_to_string( $all_access_pass->download_limit_time_period );

		$assembled_string = $all_access_pass->download_limit . ' ' . $per_string . ' ' . $time_period_string;
	}

	return apply_filters( 'edd_all_access_download_limit_string', $assembled_string, $all_access_pass );
}

/**
 * Get the string for an All Access Pass's duration. For example, if you want to show "1 year" when referring to the length of an All Access Pass.
 *
 * @since    1.0.0
 * @param    object - An All Access Pass object
 * @return   string - Easy to read string representing the duration for an All Access Pass.
 */
function edd_all_access_duration_string( $all_access_pass ){

	$duration_string = NULL;

	switch( $all_access_pass->duration_unit ){
		case 'never':
			$duration_string = __( 'Never expires', 'edd-all-access' );
			break;
		case 'year':
			$duration_string = $all_access_pass->duration_number > 1 ? __( 'years', 'edd-all-access' ) : __( 'year', 'edd-all-access' );
			break;
		case 'month':
			$duration_string = $all_access_pass->duration_number > 1 ? __( 'months', 'edd-all-access' ) : __( 'month', 'edd-all-access' );
			break;
		case 'week':
			$duration_string = $all_access_pass->duration_number > 1 ? __( 'weeks', 'edd-all-access' ) : __( 'week', 'edd-all-access' );
			break;
		case 'day':
			$duration_string = $all_access_pass->duration_number > 1 ? __( 'days', 'edd-all-access' ) : __( 'day', 'edd-all-access' );
			break;
	}


	if( 'never' == $all_access_pass->duration_unit ){
		$assembled_string = __( 'Never expires', 'edd-all-access' );
	}else{
		$assembled_string = $all_access_pass->duration_number . ' ' . $duration_string;
	}

	return apply_filters( 'edd_all_access_duration_string', $assembled_string, $all_access_pass );
}

/**
 * Get the number of time periods that have passed since the original purchase.
 * For example, if the download limit is 1 download per day, here we find the number of days since the payment took place.
 * If the download limit is 1 download per year, here we find the number of years since the payment took place.
 *
 * @since    1.0.0
 * @param    object - An All Access Pass object
 * @return   int - The number of download-limit-time-periods that have passed since the original payment.
 */
function edd_all_access_get_download_limit_time_periods_since_payment( $all_access_pass ){

	//Default periods to 0
	$periods_since_payment = 0;

	$start_date = date_create( '@' . $all_access_pass->start_time );
	$now = date_create( "now" );

	$interval = date_diff( $start_date, $now );

	$years_since_payment = $interval->y;
	$months_since_payment = $interval->m;
	$days_since_payment = $interval->d;
	$weeks_since_payment = $days_since_payment / 7;
	//$hours_since_payment = $interval->h + ( $days_since_payment * 24 );

	switch( $all_access_pass->download_limit_time_period ){
		case 'per_day':
			$periods_since_payment = $days_since_payment;
			break;
		case 'per_week':
			$periods_since_payment = $weeks_since_payment;
			break;
		case 'per_month':
			$periods_since_payment = $months_since_payment;
			break;
		case 'per_year':
			$periods_since_payment = $years_since_payment;
			break;
		case 'per_period':
			$periods_since_payment = 0;
			break;
	}

	// We "floor" this because only need to know the number of *completed* "weeks" or "days" - not half days or fractions.
	return floor( $periods_since_payment );

}

/**
 * Get the period in which the downloads-used count was last reset to 0.
 * For example, if all access was just purchased and it allows 1 download per month, the last reset period is 0 - because it was never reset.
 * If 5 months have passed but the downloads-used count was last reset to 0 in month 1, the period returned here is 1.
 *
 * @since    1.0.0
 * @param    object - An All Access Pass object
 * @return   int - The time period in which the downloads-used counter was last reset to 0.
 */
function edd_all_access_get_download_limit_last_reset_period( $all_access_pass ){

	// If the downloads-used counter has never been reset
	if ( 0 == $all_access_pass->downloads_used_last_reset ){
		return 0;
	}

	// Default to 0 periods
	$periods_between_payment_and_last_reset = 0;

	$start_date = date_create( '@' . $all_access_pass->start_time );
	$last_reset_date = date_create( '@' . $all_access_pass->downloads_used_last_reset );

	$interval = date_diff( $start_date, $last_reset_date );

	$years_between_payment_and_last_reset = $interval->y;
	$months_between_payment_and_last_reset = $interval->m;
	$days_between_payment_and_last_reset = $interval->d;
	$weeks_between_payment_and_last_reset = $days_between_payment_and_last_reset / 7;
	//$hours_between_payment_and_last_reset = $interval->h + ( $days_between_payment_and_last_reset * 24 );

	switch( $all_access_pass->download_limit_time_period ){
		case 'per_day':
			$periods_between_payment_and_last_reset = $days_between_payment_and_last_reset;
			break;
		case 'per_week':
			$periods_between_payment_and_last_reset = $weeks_between_payment_and_last_reset;
			break;
		case 'per_month':
			$periods_between_payment_and_last_reset = $months_between_payment_and_last_reset;
			break;
		case 'per_year':
			$periods_between_payment_and_last_reset = $years_between_payment_and_last_reset;
			break;
		case 'per_period':
			$periods_between_payment_and_last_reset = 0;
			break;
	}

	// We "floor" this because only need to know the number of *completed* "weeks" or "days" - not half days or fractions.
	return floor( $periods_between_payment_and_last_reset );

}

/**
 * Get the timestamp of when the previous period ended.
 * For example, if we are allowed 1 download per week and it's been 5 weeks since purchase, get the timestamp for the end of week 4.
 *
 * @since    1.0.0
 * @param    object - An All Access Pass object
 * @return   int - The timestamp for when the previous period ended.
 */
function edd_all_access_get_current_period_start_timestamp( $all_access_pass ){

	$periods_since_payment = edd_all_access_get_download_limit_time_periods_since_payment( $all_access_pass );

	switch( $all_access_pass->download_limit_time_period ){
		case 'per_day':
			$time_string = 'days';
			break;
		case 'per_week':
			$time_string = 'weeks';
			break;
		case 'per_month':
			$time_string = 'months';
			break;
		case 'per_year':
			$time_string = 'years';
			break;
		case 'per_period':
			$time_string = 0;
			break;
	}

	if( empty( $time_string ) ){
		return false;
	}

	$previous_period_timestamp = strtotime( '+' . $periods_since_payment . ' ' . $time_string, $all_access_pass->start_time );

	return $previous_period_timestamp;

}

/**
 * Check if a specific customer has a valid, specific All Access Pass.
 *
 * @since    1.0.0
 * @param    int - The ID of the user
 * @param    int - The ID of the All Access product
 * @param    int - The price_id (price variation) of the All Access product
 * @param    string - The status of the pass we want to check if the user has.
 * @return   object/int - The All Access Pass if it exists or false if not.
 */
function edd_all_access_user_has_pass( $user_id, $download_id, $price_id = 0, $required_pass_status = 'active' ){

	$has_pass = false;

	// Get the Customer
	$customer = new EDD_Customer( $user_id, true );

	// If no customer was found (perhaps not logged in)
	if ( 0 == $customer->id ){
		return $has_pass;
	}

	// Get the current customer's All Access Passes from the customer meta
	$customer_all_access_passes = $customer->get_meta( 'all_access_passes' );

	// If the customer has no All Access Passes in their customer meta
	if ( empty( $customer_all_access_passes ) || ! is_array( $customer_all_access_passes ) ){
		return $has_pass;
	}

	// Loop through each All Access Pass to see if any match the restricted_to and are active
	foreach( $customer_all_access_passes as $purchased_download_id_price_id => $purchased_aa_data ){

		if ( ! isset( $purchased_aa_data['payment_id'] ) || ! isset( $purchased_aa_data['download_id'] ) || ! isset( $purchased_aa_data['price_id'] ) ){
			continue;
		}

		// Set up an All Access Pass Object for this
		$all_access_pass = EDD_All_Access_Pass( $purchased_aa_data['payment_id'], $purchased_aa_data['download_id'], $purchased_aa_data['price_id'] );

		// If this All Access Pass matches the required status, check if it is one of the restricted_to products
		if ( $all_access_pass->status == $required_pass_status ){

			// If this All Access Pass is the one that was purchased
			if ( $download_id == $purchased_aa_data['download_id'] && $price_id == $purchased_aa_data['price_id'] ){

				// We can stop looping now because we found a valid pass with the required status.
				return $has_pass = $all_access_pass;
			}

		}else{
			$has_pass = false;
		}
	}

	return $has_pass;
}

/**
 * By passing an All Access Pass object, check if a, higher, upgraded-to version of that All Access Pass is active for the customer.
 *
 * @since    1.0.0
 * @param    object - The All Access Pass Object to check
 * @return   bool
 */
function edd_all_access_user_has_upgrade_of_prior_pass( $prior_all_access_pass ){

	// If this pass is not a "prior", it hasn't been upgraded.
	if ( ! $prior_all_access_pass->is_prior_of ){
		return false;
	}

	if ( 'upgraded' != $prior_all_access_pass->status ){
		return false;
	}

	// Get the current customer's All Access Passes from the customer meta
	$customer_all_access_passes = $prior_all_access_pass->customer->get_meta( 'all_access_passes' );

	// Loop through the customer's All Access Passes to check if the upgraded version is still active.
	foreach( $customer_all_access_passes as $purchased_download_id_price_id => $purchased_aa_data ){

		// In case there happens to be an entry in the array without a numeric key
		if ( empty( $purchased_download_id_price_id ) ){
			continue;
		}

		if ( ! isset( $purchased_aa_data['payment_id'] ) || ! isset( $purchased_aa_data['download_id'] ) || ! isset( $purchased_aa_data['price_id'] ) ){
			continue;
		}

		$all_access_pass = EDD_All_Access_Pass( $purchased_aa_data['payment_id'], $purchased_aa_data['download_id'] , $purchased_aa_data['price_id'] );

		// If this All Access Pass is still active and has been upgraded
		if ( 'active' == $all_access_pass->status && $all_access_pass->prior_all_access_passes ){

			// Loop through each of the prior All Access Passes attached to this and see if they match
			foreach( $all_access_pass->prior_all_access_passes as $prior_all_access_pass_id ){

				// If the product being purchased matches one of the prior passes in this still-active/upgraded pass
				if ( $prior_all_access_pass->id == $prior_all_access_pass_id ){
					return true;
				}
			}
		}
	}

	return false;

}

/**
 * This function will return different HTML depending on the current state of the viewer.
 * If logged out it will return HTML containing a Buy Button for an All Access Pass and a Login Form below it.
 * If logged in without an active All Access Pass, it will output a Buy Button only.
 * If logged in with a valid All Access Pass, it will simply return false, as no output is needed.
 *
 * @since       1.0.0
 * @param       array $atts The args for the output
 * @return      mixed String or Boolean. See description above.
 */
function edd_all_access_buy_or_login_form( $atts ){

	$default_atts = array(
		'all_access_download_id' => false,
		'all_access_price_id'    => false,
		'all_access_sku'         => '',
		'all_access_price'       => true,
		'all_access_direct'      => '0',
		'all_access_btn_text'    => false,
		'all_access_btn_style'   => edd_get_option( 'button_style', 'button' ),
		'all_access_btn_color'   => edd_get_option( 'checkout_color', 'blue' ),
		'all_access_btn_class'   => 'edd-submit',
		'all_access_form_id'     => '',
		'class'                  => 'edd-aa-login-purchase-normal-mode',
		'popup_login'            => true,
		'buy_instructions'       => '',
		'login_instructions'     => '',
		'login_btn_style'        => 'text',
	);
	$atts = wp_parse_args( $atts, $default_atts );

	// Set up some defaults
	$login_purchase_area_already_output = false;

	// If all_access_download_id is not an array, make it one.
	if ( ! is_array( $atts['all_access_download_id'] ) ){
		$atts['all_access_download_id'] = array( $atts['all_access_download_id'] );
	}

	// Loop through each All Access product that might be output for sale
	foreach( $atts['all_access_download_id'] as $all_access_download_id ){

		$customer_has_all_access_pass = edd_all_access_user_has_pass( get_current_user_id(), $all_access_download_id, 0 );

		// If the customer has any of the All Access passes in question, there's no output needed so return false.
		if ( $customer_has_all_access_pass ){
			return false;
		}
	}

	// If we got this far, the customer does not have any valid All Access passes of the ones required. Therefore, output is required. Lets set it up.
	if ( ! $login_purchase_area_already_output ){
		$login_purchase_area = '<div class="edd-aa-login-purchase-area ' . esc_attr( $atts['class'] ) . ' ">';
		$login_purchase_area_already_output = true;
	}

		$login_purchase_area .= '<div class="edd-aa-buy-wrapper">';
			if ( !empty( $atts['buy_instructions'] ) ){
				$login_purchase_area .= '<span class="edd-aa-buy-instructions">' . $atts['buy_instructions'] . '</span>';
			}

			$login_purchase_area .= '<div class="edd-aa-buy-btns-wrapper">';

			// Check if the site owner has chosen to show a custom link to a custom URL - like a "pricing" page:
			$all_access_replace_aa_btns_with_custom_btn = edd_get_option( 'all_access_replace_aa_btns_with_custom_btn', 'normal_aa_btns' );
			$custom_btn_url = apply_filters( 'edd_all_access_custom_url_btn_url', edd_get_option( 'all_access_custom_url_btn_url', get_bloginfo( 'wpurl' ) ) );
			$custom_btn_text = apply_filters( 'all_access_custom_url_btn_text', edd_get_option( 'all_access_custom_url_btn_text', __( 'View Pricing', 'edd-all-access' ) ) );

			// If we should show a custom button (think "pricing") instead of the All Access purchase buttons
			if( 'custom_btn' == $all_access_replace_aa_btns_with_custom_btn ){

				$class = implode( ' ', array( 'edd-all-access-btn', 'button', 'edd-button', $atts['all_access_btn_style'], $atts['all_access_btn_color'], trim( $atts['all_access_btn_class'] ) ) );

				$login_purchase_area .= '<div class="edd-aa-custom-btn-wrapper"><a href="' . esc_url( $custom_btn_url ) . '" class="' . esc_attr( $class ) . '">' . $custom_btn_text . '</a></div>';

			}else{

				// Loop through each All Access product that might be output for sale
				foreach( $atts['all_access_download_id'] as $all_access_download_id ){

					// Now we will output the purchase button. Logged in or out, we know they don't have All Access at this point.
					$all_access_buy_btn_args = array(
						'id' 	               => $all_access_download_id,
				 		'price_id'             => 0,
				 		'sku'                  => $atts['all_access_sku'],
				 		'price'                => $atts['all_access_price'],
				 		'direct'               => $atts['all_access_direct'],
				 		'text'                 => empty( $atts['all_access_btn_text'] ) ? get_the_title( $all_access_download_id ) : $atts['all_access_btn_text'],
				 		'style'                => $atts['all_access_btn_style'],
				 		'color'                => $atts['all_access_btn_color'],
				 		'class'                => $atts['all_access_btn_class'],
				 		'form_id'              => $atts['all_access_form_id'],
					);

					// Override text only if not provided / empty
					if ( ! $all_access_buy_btn_args['text'] ) {
						if( $all_access_buy_btn_args['direct'] == '1' || $all_access_buy_btn_args['direct'] == 'true' ) {
							$all_access_buy_btn_args['text'] = edd_get_option( 'buy_now_text', __( 'Buy Now', 'edd-all-access' ) );
						} else {
							$all_access_buy_btn_args['text'] = edd_get_option( 'add_to_cart_text', __( 'Purchase All Access', 'edd-all-access' ) );
						}
					}

					// Override color if color == inherit
					if( isset( $all_access_buy_btn_args['color'] )	) {
						$all_access_buy_btn_args['color'] = ( $all_access_buy_btn_args['color'] == 'inherit' ) ? '' : $all_access_buy_btn_args['color'];
					}

					if( ! empty( $all_access_buy_btn_args['sku'] ) ) {

						$download = edd_get_download_by( 'sku', $all_access_buy_btn_args['sku'] );

						if ( $download ) {
							$all_access_buy_btn_args['download_id'] = $download->ID;
						}

					} elseif( isset( $all_access_buy_btn_args['id'] ) ) {

						// Edd_get_purchase_link() expects the ID to be download_id since v1.3
						$all_access_buy_btn_args['download_id'] = $all_access_buy_btn_args['id'];

						$download = edd_get_download( $all_access_buy_btn_args['download_id'] );

					}

					if ( $download && $download->post_status == 'publish' ) {

						$login_purchase_area .= '<div class="edd-aa-buy-btn-wrapper">' . edd_get_purchase_link( $all_access_buy_btn_args ) . '</div>';

					}
				}
			}

			$login_purchase_area .= '</div>'; //End the edd-aa-buy-btns-wrapper

		$login_purchase_area .= '</div>'; // End the edd-aa-buy-wrapper

		// If this user is not logged in, output a login form/button as well.
		if ( ! is_user_logged_in() ){

            if ( ! $login_purchase_area_already_output ){
                $login_purchase_area .= '<div class="edd-aa-login-purchase-area">';
                $login_purchase_area_already_output = true;
            }

			// Show any error messages after login form submission
			edd_print_errors();

            // Show a button to login which will open the popup modal
            $login_purchase_area .= '<div class="edd-aa-login-wrapper">';
				if( !empty( $atts['login_instructions'] ) ){
			    	$login_purchase_area .= '<span class="edd-aa-login-instructions">' . $atts['login_instructions'] . '</span>';
				}

	            // If we should use the popup modal for the login
	            if ( $atts['popup_login'] == 'true' ){

	                // Allow other plugins to prevent the loading of the sw modal popup script. Free Downloads will use this to prevent a double loading of jbox.
	                $enqueue_from_all_access = apply_filters( 'edd_all_access_enqueue_jbox', true );

	                // Get a list of required scripts before enqueuing the login modal scripts
	                $required_scripts = apply_filters( 'edd_all_access_required_before_modal', array( 'jquery', 'edd-all-access-jbox' ) );

	                if ( $enqueue_from_all_access ){
	                    wp_enqueue_style( 'edd-all-access-jbox', EDD_ALL_ACCESS_URL . 'assets/js/jBox/Source/jBox.css' );
	                    wp_enqueue_script( 'edd-all-access-jbox', EDD_ALL_ACCESS_URL . 'assets/js/jBox/Source/jBox.min.js', array( 'jquery' ), EDD_ALL_ACCESS_VER, true );
	                }

	                // Use minified libraries if SCRIPT_DEBUG is turned off
	                $suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

	                // Enqueue the scripts which trigger the jbox to open
	                wp_enqueue_script( 'edd-all-access-login-modal', EDD_ALL_ACCESS_URL . 'assets/js/eddaa-login-modal' . $suffix . '.js', $required_scripts, EDD_ALL_ACCESS_VER, true );

	                // Output the login form into the footer
	                add_action( 'wp_footer', 'edd_all_access_output_login_modal' );

	                // Show a button to login which will open the popup modal
					if ( 'text' == $atts['login_btn_style'] ){
						$login_purchase_area .= ' <a href="' . apply_filters( 'edd_all_access_login_url', wp_login_url() ) . '" class="edd-aa-login-link">' .  __( 'Log In', 'edd-all-access' )  . '</a>';
					}else{
						$login_purchase_area .= ' <a href="' . apply_filters( 'edd_all_access_login_url', wp_login_url() ) . '" class="edd-aa-login-link edd-submit button">' .  __( 'Log In', 'edd-all-access' )  . '</a>';
					}

	            }else{
	                // Show the login form
	                $login_purchase_area .= edd_login_form();
	            }

            $login_purchase_area .= '</div>'; // Close edd-aa-login-wrapper

		}

	if ( $login_purchase_area_already_output ){
        $login_purchase_area .= '</div>'; // Close edd-aa-login-purchase-area
    }

	return $login_purchase_area;
}

/**
 * Hook to footer to output login form in a modal popup box in the footer
 *
 * @since       1.0.0
 * @return      void
 */
function edd_all_access_output_login_modal() {

	global $edd_all_access_output_login_modal;

	// Prevent Loading login modal twice
	if ( $edd_all_access_output_login_modal ){
		return false;
	}

	$edd_all_access_output_login_modal = true;

	echo '<div id="edd-aa-login-modal-content" style="display:none;">';
		echo edd_login_form();
	echo '</div>';

}
