<?php
/**
 * Misc Functions
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * If a customer is looking at the purchase button for an All Access Product that they have an active All Access Pass for,
 * change the word on the button from "Purchase" to "Renew".
 *
 * @since       1.0.0
 * @param       array $args The values relating to displaying the Purchase Button
 * @return      array $args The values relating to displaying the Purchase Button
 */
function edd_all_access_modify_renew_btn_text( $args ) {

	// First lets check if this is an All Access Product
	if ( ! edd_all_access_download_is_all_access( $args['download_id'], $args['price_id'] ) ){

		return $args;
	}

	// Default the price id to 0 if none is passed
	if( empty( $args['price_id'] ) ){
		$args['price_id'] = 0;
	}

	$customer = new EDD_Customer( get_current_user_id(), true );

	// Get the All Access passes saved to this customer meta
	$customer_all_access_passes = $customer->get_meta( 'all_access_passes' );

	// If this customer has no all access passes return the button as-is
	if ( empty( $customer_all_access_passes ) ){
		return $args;
	}

	// If this customer has purchased this All Access Pass before, change the button text to "Renew" to provide better context
	if ( isset( $customer_all_access_passes[$args['download_id'] . '_' . $args['price_id']] ) ){

		$show_price = $args['price'] && $args['price'] !== 'no';

		$renew_button_text = apply_filters( 'edd_all_access_renew_btn_text', __( 'Renew Now', 'edd-all-access' ), $args );

		if ( $show_price ){

			$download = new EDD_Download( $args['download_id'] );
			$variable_pricing = $download->has_variable_prices();

			if ( $variable_pricing && false !== $args['price_id'] ) {

				$price_id            = $args['price_id'];
				$prices              = $download->prices;
				$options['price_id'] = $args['price_id'];
				$found_price         = isset( $prices[$price_id] ) ? $prices[$price_id]['amount'] : false;

				if ( $show_price ) {
					$price = $found_price;
				}

			} elseif ( ! $variable_pricing ) {

				if ( $show_price ) {
					$price = $download->price;
				}

			}

			$args['text'] = edd_currency_filter( edd_format_amount( $price ) ) . '&nbsp;&ndash;&nbsp;' . $renew_button_text;
		}else{
			$args['text'] = $renew_button_text;
		}

	}

	return $args;
}
add_filter( 'edd_purchase_link_args', 'edd_all_access_modify_renew_btn_text' );

/**
 * Prevent bad purchases of All Access.
 * Make sure the user is logged in and has an account.
 * Also prevent an prior (prior-to-upgraded) All Access Pass from being purchased while the upgrade is still active.
 * This prevents errors relating to upgrades since it would overwrite the is_prior_of value.
 *
 * @since       1.0.0
 * @param       array $user
 * @param       array $valid_data
 * @param       array $post_data
 * @return      void
 */
function edd_all_access_prevent_bad_purchases( $user, $valid_data, $post ) {

	$cart_contents = edd_get_cart_contents();

	if ( ! is_array( $cart_contents ) || $user === true ){
		return false;
	}

	// Check which mode the site is in for selling All Access
	$sell_mode = edd_get_option( 'all_access_purchase_form_display', 'normal-mode' );

	// Get all of the All Access enabled products
	$all_access_products = edd_all_access_get_all_access_downloads();
	$all_access_products = array_reverse( $all_access_products );

	// Check each item in the cart
	foreach( $cart_contents as $cart_key => $item ){

		$download_id = $item['id'];
		$price_id = isset( $item['options']['price_id'] ) ? (int) $item['options']['price_id'] : 0;

		// Check if this product is an All Access product
		$enabled = get_post_meta( $download_id, '_edd_all_access_enabled', true ) ? true : false;

		// If this is not an All Access Product
		if ( ! $enabled ){

			// If the store is in the mode which only allows products to be downloaded using All Access, and the product 'could' be downloaded through All Access.
			if ( $sell_mode == 'aa-only-mode' ){

				// Loop through all of the All Access Products
				foreach( $all_access_products as $all_access_product_id ){
					//Check if this All Access Product would contain this download
					$aa_includes_download_args = array(
						'all_access_product_info' => array(
							'download_id' => $all_access_product_id,
							'price_id' => 0
						),
						'desired_product_info' => array(
							'download_id' => $download_id,
							'price_id' => $price_id,
						)
					);

					$all_access_includes_download = edd_all_access_includes_download( $aa_includes_download_args );

					// Set the name of the forbidden product
					$forbidden_product_title = get_the_title( $download_id );
					if ( $price_id != 0 ){
						$variable_prices = edd_get_variable_prices( $download_id );
						$forbidden_product_title .= ' - ' . $variable_prices[$price_id]['name'];
					}

					if( $all_access_includes_download['success'] ){

						// Remove the item from the cart
						$cart_object = new EDD_Cart();
						$cart_object->remove( $cart_key );

						// Show an error why
						edd_set_error( 'edd_all_access_purchase_not_possible', '"' . $forbidden_product_title . '": ' . __( 'You can not purchase this product directly. To get access, purchase', 'edd-all-access' ) . ' <a href="' . get_permalink( $all_access_product_id ) . '">"' . get_the_title( $all_access_product_id ) . '"</a>.' );
					}

				}

			}else{
				// This product is not covered by all access. Allow it to be purchased normally.
				continue;
			}

			return false;
		}

		// If we got to here, this is an All Access enabled product that is in the cart

		$bypass_login_requirement = apply_filters( 'edd_all_access_bypass_login_requirement', false, $download_id, $price_id );

		// Check if the customer is not logged in
		if ( ! is_user_logged_in() && ! $bypass_login_requirement ){
			edd_set_error( 'edd_all_access_purchase_not_possible', __( 'You must be logged-in to purchase', 'edd-all-access' ) . ' ' . get_the_title( $download_id ) . '.' );
		}

		// Check the quantity of the All Access product. If it's more than 1, at this time, you can't buy more than 1 quantity of an All Access Pass in a single purchase.
		// This is because you can only buy an access pass for yourself (your own account). You can't purchase for other accounts at this time.
		if ( isset( $item['quantity'] ) && $item['quantity'] > 1 ){
			edd_set_error( 'edd_all_access_quantity_error', sprintf( __( 'You can only purchase 1 "%s" per user account. If you wish to purchase for another user account, log into that account and purchase it there as well. To complete this purchase, reduce the quantity in your cart to 1.', 'edd-all-access' ), get_the_title( $download_id ) ) );
		}

		// Different than official "quantities", but if there is more than 1 of the same All Access product in the cart, throw an error
		foreach( $cart_contents as $checker_cart_key => $checker_item ){

			// Skip over this item of course - we only want to check if OTHER products in the cart match this one.
			if ( $checker_cart_key == $cart_key ){
				continue;
			}

			// If this item is in the cart more than once
			if ( $checker_item['id'] == $item['id'] ){
				edd_set_error( 'edd_all_access_quantity_error', sprintf( __( 'You can only purchase 1 "%s" per user account. If you wish to purchase for another user account, log into that account and purchase it there as well. To complete this purchase, reduce the quantity in your cart to 1.', 'edd-all-access' ), get_the_title( $download_id ) ) );
				break;
			}
		}

		$has_active_all_access_pass = edd_all_access_user_has_pass( get_current_user_id(), $download_id, $price_id, 'active' );
		$has_upgraded_all_access_pass = edd_all_access_user_has_pass( get_current_user_id(), $download_id, $price_id, 'upgraded' );

		// Check if the customer already has an All Access Pass for this product
		if ( $has_active_all_access_pass ){

			// Check if this All Access Pass is set to never expire. If so, there's no need to purchase it again.
			if( 'never' == $has_active_all_access_pass->expiration_time ){
				edd_set_error( 'edd_all_access_purchase_not_possible', sprintf( __( 'You already own %s and it is set to never expire.', 'edd-all-access' ), get_the_title( $download_id ) ) );
			}

			// If it actually will expire at some point, allow it to be renewed. No errors here!
			continue;
		}

		if( $has_upgraded_all_access_pass instanceof EDD_All_Access_Pass ){
			$all_access_pass = $has_upgraded_all_access_pass;

			// If the customer has an active & upgraded version of this pass already
			if ( edd_all_access_user_has_upgrade_of_prior_pass( $all_access_pass ) ){
				edd_set_error( 'edd_all_access_purchase_not_possible', __( 'You already own the upgraded version of', 'edd-all-access' ) . ' ' . get_the_title( $download_id ) );
			}
		}
	}
}
add_action( 'edd_checkout_user_error_checks', 'edd_all_access_prevent_bad_purchases', 10, 3 );

/**
 * Automatically remove deuplciate All Access products from the cart
 *
 * @since       1.0.0
 * @param       array $cart The array of items in the cart
 * @return      array $cart The array of items in the cart
 */
function edd_all_access_auto_remove_duplicates( $cart ){

	$all_access_products = edd_all_access_get_all_access_downloads();

	$in_cart_aa_products = array();

	// Loop through all the items in the cart
	foreach ( $cart as $key => $item ) {
		$download = new EDD_Download( $item['id'] );

		// If the item is an All Access product
		if ( in_array( $download->ID, $all_access_products ) ) {

			// If this item is already in the cart
			if ( in_array( $download->ID, $in_cart_aa_products ) ){

				//Remove it from the cart because it is a duplicate All Access product.
				unset( $cart[ $key ] );

			}else{

				// Store it in the list of "currently in the cart" all access products.
				$in_cart_aa_products[] = $download->ID;
			}

		}
	}

	return $cart;

}
add_filter( 'edd_cart_contents', 'edd_all_access_auto_remove_duplicates', 10, 1 );

/**
 * Modify the purchase form (Add To Cart) to append the Buy/Login form for All Access.
 * The default here is to make no change to the normal purchase form but if set in the settings, the output will change.
 *
 * @since       1.0.0
 * @param       string $form The existing download form
 * @param       array $args Arguments passed to the form
 * @return      string $form The updated download form
 */
function edd_all_access_modify_purchase_form( $purchase_form, $args ) {

	// If we should only show the All Access purchase/login form, find which AA pass could contain this product
	$all_access_products = edd_all_access_get_all_access_downloads();
	$all_access_products = array_reverse( $all_access_products );

	// Don't run this if the product is an All Access Product.
	if ( in_array( $args['download_id'], $all_access_products ) ){
		return $purchase_form;
	}

	// Set default output for the purchase form
	$form = '';

	// Set the download and price ids
	$download_id = absint( $args['download_id'] );
	$price_id = defined( 'DOING_AJAX' ) && DOING_AJAX ? absint( $args['price_id'] ) : edd_get_default_variable_price( $download_id );

	// Don't run this if the customer has access to this product with their current All Access Pass
	$all_access_check_args = array(
		'download_id' => $download_id,
		'price_id' => $price_id
	);

	$all_access = edd_all_access_check( $all_access_check_args );

	// If the customer has access to this product through an All Access Pass, return the purchase form as is.
	if ( $all_access['success'] ){
		return $purchase_form;
	}

	// Check if this product being viewed has variable pricing
	$has_variable_prices = edd_has_variable_prices( $download_id );

	// Check if any of the variable prices exclude All Access.
	$variable_prices = edd_get_variable_prices( $download_id );

	// If this product has variable pricing and any of those variable prices are not covered by an All Access Pass, return the normal purchase form.
	if ( $has_variable_prices ) {

		// Loop through each price id to check if it is excluded from All Access
		foreach( $variable_prices as $variable_price_id => $variable_price_settings ){
			// If this variable price is excluded from All Access
			if( ! empty( $variable_price_settings[ 'excluded_price' ] ) ){
					return $purchase_form;
			}
		}

	}

	// Get the mode the user has chosen for the Add To Cart purchase form display in the All Access settings:
	$purchase_form_display_mode = edd_get_option( 'all_access_purchase_form_display', 'normal-mode' );

	// If the site is set to use the normal purchase mode
	if( 'normal-mode' == $purchase_form_display_mode ){

		// Add the normal Buy Now form back.
		return $purchase_form;

	}elseif( 'normal-plus-aa-mode' == $purchase_form_display_mode || 'aa-only-mode' == $purchase_form_display_mode ){

		// Set default
		$all_access_includes_download = false;
		$every_price_id_included = true;
		$all_access_products_to_show = array();

		// Loop through each All Access Product
		foreach( $all_access_products as $all_access_product_id ){

			// If this product has variable pricing and any of those variable prices are not covered by an All Access Pass, return the normal purchase form.
			if ( $has_variable_prices ) {

				// Loop through each price id to check if it is excluded from this All Access Product
				foreach( $variable_prices as $variable_price_id => $variable_price_settings ){

					//Check if this All Access Pass would contain this download/price_id
					$aa_includes_download_args = array(
						'all_access_product_info' => array(
							'download_id' => $all_access_product_id,
							'price_id' => 0
						),
						'desired_product_info' => array(
							'download_id' => $download_id,
							'price_id' => $variable_price_id,
						)
					);

					$all_access_includes_download = edd_all_access_includes_download( $aa_includes_download_args );

					// If this All Access Pass does not cover every price id in the product being viewed.
					if ( ! $all_access_includes_download['success'] ){

						$every_price_id_included = false;
						// No need to check any more price ids
						break;
					}
				}

				// If this All Access pass failed to cover every price id
				if ( ! $every_price_id_included ){
					// Reset the value for the next All Access Pass to check
					$every_price_id_included = true;
					// And then skip this pass and start checking the next one.
					continue;
				}else{

					// If it would contain this download, we'll output a link to buy the All Access Pass
					if ( $all_access_includes_download['success'] ){

						// Add this All Access product to the list of All Access Products we'll show links to.
						$all_access_products_to_show[] = $all_access_product_id;
					}
				}
			}else{

				// Single price mode (not variable)

				//Check if this All Access Pass would contain this download
				$aa_includes_download_args = array(
					'all_access_product_info' => array(
						'download_id' => $all_access_product_id,
						'price_id' => 0
					),
					'desired_product_info' => array(
						'download_id' => $download_id,
						'price_id' => $price_id,
					)
				);

				$all_access_includes_download = edd_all_access_includes_download( $aa_includes_download_args );

				// If it would contain this download, we'll output a link to buy the All Access Pass
				if ( $all_access_includes_download['success'] ){

					// Add this All Access product to the list of All Access Products we'll show links to.
					$all_access_products_to_show[] = $all_access_product_id;
				}
			}
		}

		// If no All Access Passes would contain this download, return the normal purchase form.
		if( empty( $all_access_products_to_show ) ){
			return $purchase_form;
		}

		// Override color if color == inherit
		if( isset( $args['color'] )	) {
			$args['color'] = 'inherit' == $args['color'] ? '' : $args['color'];
		}

		// Check if we should show the buy instructions
		$show_buy_instructions = edd_get_option( 'all_access_show_buy_instructions', 'show' );
		$buy_instructions = edd_get_option( 'all_access_buy_instructions', __( 'To get access, purchase an All Access Pass here.', 'edd-all-access' ) );

		// Check if we should show the login instructions
		$show_login_instructions = edd_get_option( 'all_access_show_login_instructions', 'show' );
		$login_instructions = edd_get_option( 'all_access_login_instructions', __( 'Already purchased?', 'edd-all-access' ) );

		$all_access_form_args = array(
			'all_access_download_id' => $all_access_products_to_show,
			'all_access_price_id'    => 0,
			'all_access_price'       => true,
			'all_access_style'       => $args['style'],
			'all_access_color'       => $args['color'],
			'all_access_class'       => $args['class'],
			'class'                  => 'edd-aa-login-purchase-' . $purchase_form_display_mode,
			'buy_instructions'       => $show_buy_instructions == 'show' ? $buy_instructions : '',
			'login_instructions'     => $show_login_instructions == 'show' ? $login_instructions : '',
		);

		// Return the All Access Purchase/Login form only
		$form = edd_all_access_buy_or_login_form( $all_access_form_args );

		//If we should show both the normal "Add To Cart" button and the All Access purchase/login form
		if( 'normal-plus-aa-mode' == $purchase_form_display_mode ){

			// Add the purchase back before the newly renovated aa-only style form.
			$form = $purchase_form . $form;
		}

	}

	// If, for any reason, the form is empty at this point, return the normal purchase form untouched.
	if ( empty( $form ) ){
		return $purchase_form;
	}

	return $form;
}
add_filter( 'edd_purchase_download_form', 'edd_all_access_modify_purchase_form', 10, 2 );

/**
 * Exclude Bundles From All Access. Bundles aren't "real" products with actual downloadable files. Only products WITHIN bundles are.
 *
 * @since       1.0.0
 * @param       bool $allowed Whether to allow All Access to change the download form for this product or not.
 * @param       string $purchase_form The actual form output which is being filtered.
 * @param       array $args The arguments passed to the edd_purchase_download_form filter.
 * @param       int $download_id The ID of the product which is being shown.
 * @return      bool $allowed Whether to allow All Access to change the download form for this product or not.
 */
function edd_all_access_exclude_bundles( $allowed, $purchase_form, $args, $download_id, $price_id ){

	$download         = new EDD_Download( $download_id );
	$type             = $download->get_type();

	// If the product being viewed is a bundle, exclude it from All Access
	if ( 'bundle' == $type ){
		return false;
	}

	return $allowed;

}
add_filter( 'edd_all_access_allow', 'edd_all_access_exclude_bundles', 10, 5 );

/**
 * Add an informational box for All Access to the View Order Details screen.
 *
 * @since       1.0.0
 * @param       int $payment_id The ID of the Payment being viewed.
 * @return      void
 */
function edd_all_access_view_order_details_sidebar( $payment_id ){

	$payment = new EDD_Payment( $payment_id );

	$at_least_one_all_access_pass_purchased = false;

	?>
	<div id="edd-order-logs" class="postbox edd-order-logs">

		<h3 class="hndle">
			<span><?php _e( 'All Access Passes', 'edd-all-access' ); ?></span>
		</h3>
		<div class="inside">
			<div class="edd-admin-box">
				<div class="edd-admin-box-inside">
					<ul class="edd-aa-view-order-details">
					<?php // Loop through each All Access Product that was purchased in this payment and list links to view/manage them
					foreach( $payment->cart_details as $cart_key => $cart_item ){

						$download_id = $cart_item['id'];

						// Check if this product is an All Access product
						$all_access_enabled = get_post_meta( $download_id, '_edd_all_access_enabled', true ) ? true : false;

						// If not an All Access product, skip it
						if( ! $all_access_enabled ){
							continue;
						}

						// Get the purchased price ID
						$price_id = isset( $cart_item['item_number']['options']['price_id'] ) ? $cart_item['item_number']['options']['price_id'] : 0;

						$product_title = get_the_title( $download_id );

						if ( edd_has_variable_prices( $download_id ) && isset( $price_id ) ) {
							$product_title .= ' - ' . edd_get_price_option_name( $download_id, $price_id, $payment_id );
						}

						// Set up the All Access Pass Object for this product
						$all_access_pass = new EDD_All_Access_Pass( $payment->ID, $download_id, $price_id );

						$at_least_one_all_access_pass_purchased = true;
						?>
						<li class="edd-aa-single-pass-payment-sidebar">
							<p>
								<span class="label"><?php echo $product_title; ?></span>&nbsp;
								<?php // If this All Access pass is invalid, don't show a link to view it

								if( 'pending' == $payment->status ){?>
									<span><?php _e( '(Payment Pending)', 'edd-all-access' ); ?></span>
								<?php }elseif( 'invalid' == $all_access_pass->status ){ ?>
									<span>(<?php echo edd_all_access_get_status_label( $all_access_pass->status ); ?>)</span>
								<?php }else{ ?>
									<span><a href="<?php echo esc_url( admin_url( 'edit.php?post_type=download&page=edd-all-access-pass&payment_id=' . $payment->ID . '&download_id=' . $download_id . '&price_id=' . $price_id ) ); ?>"><?php _e( '(View Details)', 'edd-all-access' ); ?></a></span>
								<?php } ?>
							</p>
						</li>
					<?php }

					// If no All Access Products were purchased, show "none".
					if ( ! $at_least_one_all_access_pass_purchased ){
						?><li class="edd-aa-single-pass-payment-sidebar">
							<?php echo __( 'None in this payment', 'edd-all-access' ); ?>
						</li>
					<?php } ?>
					</ul>
				</div>
			</div><!-- /.column-container -->

		</div><!-- /.inside -->

	</div><!-- /#edd-order-logs --><?php
}
add_action( 'edd_view_order_details_sidebar_after', 'edd_all_access_view_order_details_sidebar' );

/**
 * Because of a bug, when using the retroactive All Access Passes tool we'll do a check to make sure
 * all data that should exist does exist prior to attempting an activation.
 * Otherwise, it's possible that the activation gets falsely triggered as a renewal. See issue #152 on GitHub for more.
 *
 * @since       1.0.7
 * @param       obj $all_access_pass The All Access Pass in question
 * @return      void
 */
function all_access_issue_152_check( $all_access_pass, $payment_id, $download_id, $price_id = 0 ){

	$price_id = empty( $price_id ) ? 0 : $price_id;

	$payment = new EDD_Payment( $payment_id );
	$customer = new EDD_Customer( $payment->customer_id );

	// Get customer meta
	$customer_all_access_passes = $customer->get_meta( 'all_access_passes' );

	// Get payment meta
	$all_access_active_ids = $payment->get_meta( '_edd_aa_active_ids', true );
	$all_access_expired_ids = $payment->get_meta( '_edd_aa_expired_ids', true );
	$purchased_aa_download_key = $download_id . '-' . $price_id;

	// Lets assume the pass was activated correctly and then double check
	$correctly_activated = true;

	// If this All Access pass does not exists in the customer meta array
	if ( ! isset( $customer_all_access_passes[$download_id . '_' . $price_id] ) ) {

		// And it's also not set in the payment meta, this All Access pass was not affected by the issue.
		if ( ! isset( $all_access_active_ids[$purchased_aa_download_key] ) && ! isset( $all_access_expired_ids[$purchased_aa_download_key] ) ){
			return;
		}
	}

	// If this All Access pass exists in either the active or expired payment meta
	if ( isset( $all_access_active_ids[$purchased_aa_download_key] ) || isset( $all_access_expired_ids[$purchased_aa_download_key] ) ){

		// But it's NOT in the customer meta, something is broken with it.
		if ( ! isset( $customer_all_access_passes[$download_id . '_' . $price_id] ) ){
			$correctly_activated = false;
		}

	}

	if ( isset( $customer_all_access_passes[$download_id . '_' . $price_id] ) ){
		$this_pass_meta = $customer_all_access_passes[$download_id . '_' . $price_id];

		//Double check that it contains all data it is expected to have
		if ( ! isset( $this_pass_meta['download_id'] ) ){
			$correctly_activated = false;
		}
		if ( ! isset( $this_pass_meta['price_id'] ) ){
			$correctly_activated = false;
		}
		if ( ! isset( $this_pass_meta['payment_id'] ) ){
			$correctly_activated = false;
		}
	}

	// If this pass was not properly activated, it needs to be deleted so that we can properly re-activate it below
	if ( ! $correctly_activated ){

		// Remove the customer meta for this pass.
		if ( isset( $customer_all_access_passes[$download_id . '_' . $price_id] ) ){
			unset( $customer_all_access_passes[$download_id . '_' . $price_id] );
		}

		// Save the updated data
		$customer->update_meta( 'all_access_passes', $customer_all_access_passes );

		// Remove the id of this download from the payment's meta
		$all_access_active_ids = $payment->get_meta( '_edd_aa_active_ids', true );
		$all_access_expired_ids = $payment->get_meta( '_edd_aa_expired_ids', true );
		$purchased_aa_download_key = $download_id . '-' . $price_id;

		if ( isset( $all_access_active_ids[$purchased_aa_download_key] ) ){
			unset( $all_access_active_ids[$purchased_aa_download_key] );
			$payment->update_meta( '_edd_aa_active_ids', $all_access_active_ids );
		}

		if ( isset( $all_access_expired_ids[$purchased_aa_download_key] ) ){
			unset( $all_access_expired_ids[$purchased_aa_download_key] );
			$payment->update_meta( '_edd_aa_expired_ids', $all_access_expired_ids );
		}
	}

}
