<?php
/**
 * Upgrade functions for All Access.
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Add the "page" which WordPress will use to handle the upgrades routines.
 *
 * @since       1.0.0
 * @param       void
 * @return      void
 */
function edd_all_access_register_upgrades_page() {

	if( ! function_exists( 'EDD' ) ) {
		return;
	}

	add_submenu_page( null, __( 'EDD All Access Upgrades', 'edd-all-access' ), __( 'EDD Upgrades', 'edd-all-access' ), 'install_plugins', 'edd-aa-upgrades', 'edd_all_access_upgrades_screen' );
}
add_action( 'admin_menu', 'edd_all_access_register_upgrades_page', 10 );

/**
 * This function controls what is shown on the upgrades page for All Access
 *
 * @since       1.0.0
 * @param       void
 * @return      void
 */
function edd_all_access_upgrades_screen() {
	$step = isset( $_GET['step'] ) ? absint( $_GET['step'] ) : 1;
	$total_steps = isset( $_GET['total_steps'] ) ? absint( $_GET['total_steps'] ) : __( 'unknown', 'edd-all-access' );
?>
	<div class="wrap">
		<h2><?php _e( 'All Access - Upgrades', 'edd-all-access' ); ?></h2>
		<div id="edd-upgrade-status">
			<p><?php _e( 'The upgrade process is running, please be patient.', 'edd-all-access' ); ?></p>
			<p><strong><?php printf( __( 'Step %d of approximately %d running', 'edd-all-access' ), $step, $total_steps ); ?>
		</div>
		<script type="text/javascript">
			document.location.href = "index.php?edd_action=<?php echo esc_html( $_GET['edd_upgrade'] ); ?>&step=<?php echo absint( $_GET['step'] ); ?>";
		</script>
	</div>
<?php
}

/**
 * Triggers all upgrade functions
 *
 * @since       1.0.0
 * @param       void
 * @return      void
 */
function edd_all_access_show_upgrade_notice() {

	if( ! function_exists( 'EDD' ) ) {
		return;
	}

	$edd_all_access_version = get_option( 'edd_all_access_version' );
	$edd_all_access_version = empty( $edd_all_access_version ) ? EDD_ALL_ACCESS_VER : $edd_all_access_version;

	if( function_exists( 'edd_has_upgrade_completed' ) && function_exists( 'edd_maybe_resume_upgrade' ) ) {
		$resume_upgrade = edd_maybe_resume_upgrade();
		if ( empty( $resume_upgrade ) ) {

			// If the customer meta data re-organization upgrades for v1 have not yet been completed, show a button in the admin to do them now.
			if ( ! isset( $_GET['edd_upgrade'] ) && version_compare( $edd_all_access_version, '1', '>=' ) && ! edd_has_upgrade_completed( 'aa_v1_reorganize_customer_meta' ) ) {
				printf(
					'<div class="updated"><p>' . __( 'The customer data needs to be upgraded for All Access, click <a href="%s">here</a> to start the upgrade.', 'edd-all-access' ) . '</p></div>',
					esc_url( add_query_arg( array( 'edd_action' => 'aa_v1_reorganize_customer_meta' ), admin_url() ) )
				);
			}
		}
	}
}
add_action( 'admin_notices', 'edd_all_access_show_upgrade_notice' );

/**
 * Upgrades old license keys with the new site URL store
 *
 * @since 1.0
 * @return void
 */
function edd_all_access_v1_upgrades_callback() {

	ignore_user_abort( true );

	if ( ! edd_is_func_disabled( 'set_time_limit' ) ) {
		set_time_limit( 0 );
	}

	$step   = isset( $_GET['step'] ) ? absint( $_GET['step'] ) : 1;
	$payments_per_page = 50;

	$args   = array(
		'number' => $payments_per_page,
		'page'   => $step,
		'status' => 'any',
		'order'  => 'ASC',
		'download' => edd_all_access_get_all_access_downloads()
	);

	$payments = new EDD_Payments_Query( $args );
	$payments = $payments->get_payments();

	if( $payments ) {

		foreach( $payments as $payment ) {

			/**
			 * Prior to initial release, customer all access data was stored using the [download_id][price_id] = array( all aa data here).
			 * Now, the download and price id are combined.
			 * In order to prevent breaking beta tester sites, we will manually re-organize the data here for all customer passes.
			 */
			$customer = new EDD_Customer( $payment->customer_id );

			// Get the All Access passes saved to this customer meta
			$customer_all_access_passes = $customer->get_meta( 'all_access_passes' );

			// If this customer has no all access data, we don't need to do anything for them so skip this customer
			if ( empty( $customer_all_access_passes ) ){
				continue;
			}

			// Loop through each All Access Pass saved to the customer meta
			foreach( $customer_all_access_passes as $aa_pass_key => $aa_datas ){

				// If one of the all access passes does not have a download and price id saved directly them them, they are old data and need to be updated.
				if ( ! isset( $aa_datas['download_id'] ) || ! isset( $aa_datas['price_id'] ) ){

					foreach( $aa_datas as $price_id => $aa_data ){

							// First lets store the data in the new key
							$customer_all_access_passes[$aa_pass_key . '_' . $price_id] = $aa_data;

							// Then lets add the download id
							$customer_all_access_passes[$aa_pass_key . '_' . $price_id]['download_id'] = $aa_pass_key;

							// And the price id
							$customer_all_access_passes[$aa_pass_key . '_' . $price_id]['price_id'] = $price_id;

							// Then we'll strip out the old data
							unset( $customer_all_access_passes[$aa_pass_key] );
					}
				}

			}

			$customer->update_meta( 'all_access_passes', $customer_all_access_passes );

		}

		// Customers with All Access data found so upgrade them
		$step++;
		$redirect = add_query_arg( array(
			'page'        => 'edd-aa-upgrades',
			'edd_upgrade' => 'aa_v1_reorganize_customer_meta',
			'step'        => $step,
			'total_steps' => count( $payments ), // Note that this total is innacurate because we can't currently do $payments->found_posts/$payments_per_page,
		), admin_url( 'index.php' ) );
		wp_safe_redirect( $redirect ); exit;

	} else {

		// No more customers found, update the DB version and finish up
		edd_set_upgrade_complete( 'aa_v1_reorganize_customer_meta' );
		wp_redirect( admin_url() ); exit;
	}

}
add_action( 'edd_aa_v1_reorganize_customer_meta', 'edd_all_access_v1_upgrades_callback' );
