<?php
/**
 * Integration functions to make All Access compatible with the EDD Content Restriction extension
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Integrates EDD All Access with the EDD Content Restriction extension
 *
 * @since 1.0.0
 */
class EDD_All_Access_Content_Restriction {

	/**
	 * Get things started
	 *
	 * @since  1.0.0
	 * @return void
	 */
	public function __construct() {

		if ( ! class_exists( 'EDD_Content_Restriction' ) ) {
			return;
		}

		add_action( 'admin_notices', array( $this, 'edd_all_access_cr_too_old_notice' ) );
		add_filter( 'edd_cr_user_can_access', array( $this, 'can_access_content' ), 10, 3 );

	}

	/**
	 * Make sure we are at the minimum version of Content Restriction - which is 2.0
	 *
	 * @since       1.0.0
	 */
	public function edd_all_access_cr_too_old_notice(){

		if ( defined( 'EDD_CONTENT_RESTRICTION_VER' ) && version_compare( EDD_CONTENT_RESTRICTION_VER, '2.0' ) <= 0 ){
			?>
			<div class="notice notice-error">
				<p><?php echo __( 'EDD All Access: Your version of EDD Content Restriction must be updated to version 2.0 or later to use the All Access extension in conjunction with Content Restriction.', 'edd-all-access' ); ?></p>
			</div>
			<?php
		}
	}

	/**
	 * Check if user has access to content
	 *
	 * @since  1.0.0
	 * @return bool
	 */
	public function can_access_content( $has_access, $user_id, $restricted_to ) {

		if( $has_access && is_array( $restricted_to ) ) {

			$restricted_to_download_ids = wp_list_pluck( $restricted_to, 'download' );
			$all_access_products = edd_all_access_get_all_access_downloads();

			$restricted_to_all_access_product = false;

			// If any of the products are NOT All Access Passes, they will override whatever All Access does anyway so there's no need to do anything else
			// Because we already know has_access is true for one of those.
			foreach( $restricted_to_download_ids as $restricted_to_download_id ){
				if ( in_array( $restricted_to_download_id, $all_access_products ) ){
					$restricted_to_all_access_product = true;
				}
			}

			if ( ! $restricted_to_all_access_product ){
				return $has_access;
			}

			// Get the Customer
			$customer = new EDD_Customer( $user_id, true );

			// Get the current customer's All Access Passes from the customer meta
			$customer_all_access_passes = $customer->get_meta( 'all_access_passes' );

			// Loop through each All Access Pass to see if any match the restricted_to and are active
			foreach( $customer_all_access_passes as $download_id_price_id => $all_access_pass_data ){

				// Set up an All Access Pass Object for this
				$all_access_pass = new EDD_All_Access_Pass( $all_access_pass_data['payment_id'], $all_access_pass_data['download_id'], $all_access_pass_data['price_id'] );

				// If this All Access Pass is active, check if it is one of the restricted_to products
				if ( $all_access_pass->status == 'active' ){

					foreach( $restricted_to as $restricted_to_product ){

						if ( ! isset( $restricted_to_product['price_id'] ) ){
							$restricted_to_product['price_id'] = 0;
						}

						// If this post is restricted to the same product this All Access Pass is tied to
						if ( $restricted_to_product['download'] == $all_access_pass_data['download_id'] && $restricted_to_product['price_id'] == $all_access_pass_data['price_id'] ){

							// We can stop looping now because we found a pass that gives access
							return $has_access = true;

						}
					}
				}else{
					$has_access = false;
				}
			}

			$has_access = false;
		}

		return $has_access;
	}



}
