<?php
/**
* Integration functions to make All Access compatible with EDD Recurring.
* How it works: If an All Access pass is set to sync with Recurring's expiration, it will expire when the customer's recurring renews.
* Note that the All Access still actually expires (so expiration functions still fire - like commission payouts).
* The renewal payment actually starts a new All Access period - though the user wouldn't notice anything changing on their end.
*
* Changing a renewal period: If you decide to edit the renewal period of your recurring All Access product, existing customers with existing subscriptions will be "grandfathered".
* This means their All Access periods will still expire with their subscriptions - as EDD recurring also grandfathers in existing subscriptions. They do not change if you edit the recurring period.
* Any NEW purchases after editing the product's recurring period will use that new recurring/expiration period for both All Access and recurring.
*
* You can use this to sell All Access passes with variable prices that renew at different times. For example, a variable price for monthly or yearly.
*
* @package     EDD\EDDAllAccess\Functions
* @since       1.0.0
*/

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
* Integrates EDD All Access with the EDD Recurring extension
*
* @since 1.0.0
*/
class EDD_All_Access_Recurring {

	/**
	* Get things started
	*
	* @since  1.0.0
	* @return void
	*/
	public function __construct() {

		if ( ! class_exists( 'EDD_Recurring' ) ) {
			return;
		}

		// Include the "Sync with Recurring expiration" option in the Post Meta for All Access duration.
		add_filter( 'edd_all_access_duration_unit_metabox_options',  array( $this, 'edd_all_access_recurring_duration_option' ) );

		// Include the "Sync with Recurring expiration" option in the Customer Meta for an All Access Pass's duration.
		add_filter( 'edd_all_access_duration_unit_customer_options',  array( $this, 'edd_all_access_recurring_duration_customer_option' ), 10, 4 );

		// Include 'edd_subscription' in the array of acceptable payment statuses for All Access
		add_filter( 'edd_all_access_valid_statuses', array( $this, 'edd_all_access_recurring_valid_payment_statuses' ) );

		// Filter the expiration time function so that All Access products set to sync with recurring expire when they should.
		add_filter( 'edd_all_access_get_expiration_time', array( $this, 'sync_expiration_time_with_recurring' ), 10, 2 );

		// Filter the duration String used to represent the duration. For example, this is used in the edd_all_access_passes shortcode to display "1 Year" for All Access Pass Duration.
		add_filter( 'edd_all_access_duration_string', array( $this, 'duration_string' ), 10, 2 );

		// Fire the expiration function for All Access Passes
		add_action( 'edd_subscription_post_renew', array( $this, 'check_expirations_post_renew' ), 10, 3 );

		// Prevent manual early renewals for All Access Passes if an active subscription exists.
		add_action( 'edd_checkout_error_checks', array( $this, 'prevent_manual_renewals' ), 10, 2 );

		add_action( 'edd_all_access_rcp_migrate_subscription', array( $this, 'migrate_rcp_subscriptions' ), 10, 3 );

	}

	/**
	* If someone is migrating their users from Restrict Content Pro to EDD All Access, here's we'll migrate their subscription to EDD Recurring
	*
	* @since    1.0.0
	* @param    string $subscription_profile_id The profile id for the subscription being migrated.
	* @param    string $edd_payment The EDD Payment which will be used as the parent payment.
	* @return   void
	*/
	public function migrate_rcp_subscriptions( $subscription_profile_id, $edd_payment, $all_access_product_id ){

		$recurring = EDD_Recurring()->is_recurring( $all_access_product_id );

		// If recurring is not enabled for the All Access product we are migrating to, don't migrate/create the subscription
		if ( ! $recurring ){
			return;
		}

		$times     = EDD_Recurring()->get_times_single( $all_access_product_id );
		$period    = EDD_Recurring()->get_period_single( $all_access_product_id );
		$has_trial  = EDD_Recurring()->has_free_trial( $all_access_product_id );
		$trial_period = EDD_Recurring()->get_trial_period( $all_access_product_id );
		$signup_fee = EDD_Recurring()->get_signup_fee_single( $all_access_product_id );

		$subscriber = new EDD_Recurring_Subscriber( $edd_payment->customer_id );

		// Set up the details for the new EDD subscription
		$args = array(
			'expiration'        => $subscriber->get_new_expiration( $all_access_product_id, 0, $trial_period ),
			'created'           => $edd_payment->date,
			'status'            => 'active',
			'profile_id'        => $subscription_profile_id,
			'transaction_id'    => $edd_payment->transaction_id,
			'initial_amount'    => $edd_payment->total,
			'recurring_amount'  => $edd_payment->total,
			'bill_times'        => $times,
			'period'            => $period,
			'parent_payment_id' => $edd_payment->ID,
			'product_id'        => $all_access_product_id,
			'customer_id'       => $edd_payment->customer_id
		);

		// Set up the subscription object
		$new_subscription = new EDD_Subscription();
		$new_subscription->create( $args );

		// Let the edd payment know it is a recurring one
		$edd_payment->update_meta( '_edd_subscription_payment', true );

	}

	/**
	* Add "Sync with Recurring expiration" as an expiration option for All Access.
	*
	* @since    1.0.0
	* @param    array $all_access_length_options Array of expiration options for All Access
	* @return   array $all_access_length_options Array (modified) of expiration options for All Access
	*/
	public function edd_all_access_recurring_duration_option( $all_access_length_options ){
		// Add an option to sync the expiration with EDD Recurring
		$all_access_length_options['edd_recurring'] = __( 'Sync with Recurring expiration', 'edd-all-access' );

		return $all_access_length_options;
	}

	/**
	* Add "Sync with Recurring expiration" as an expiration option for All Access on the Customer Meta (Single All Access Pass Page).
	*
	* @since    1.0.0
	* @param    array $all_access_length_options Array of expiration options for All Access
	* @return   array $all_access_length_options Array (modified) of expiration options for All Access
	*/
	public function edd_all_access_recurring_duration_customer_option( $all_access_length_options, $payment, $download_id, $price_id ){

		// Check if this payment has a subscription attached to it
		$payment_has_subscription = edd_get_payment_meta( $payment->ID, '_edd_subscription_payment' );

		if ( ! $payment_has_subscription ){
			// Add an option to sync the expiration with EDD Recurring
			$all_access_length_options['edd_recurring'] = __( 'Never Expires (Sync with Recurring expiration).', 'edd-all-access' );
		}else{
			// Add an option to sync the expiration with EDD Recurring
			$all_access_length_options['edd_recurring'] = __( 'Sync with Recurring expiration', 'edd-all-access' );
		}

		return $all_access_length_options;
	}

	/**
	* Include valid payment statuses to the list of valid statuses
	*
	* @since    1.0.0
	* @param    array - $valid_payment_statuses
	* @return   array - $valid_payment_statuses
	*/
	public function edd_all_access_recurring_valid_payment_statuses( $valid_payment_statuses ){
		$valid_payment_statuses['edd_subscription'] = 'edd_subscription';
		return $valid_payment_statuses;
	}

	/**
	* Make All Access pass payments that are set to sync with Recurring expiration expire when the Recurring does.
	* Note that EDD Recurring "grandfathers" existing subscriptions if the period for the product is updated.
	* Thus, we need to get the period from the customer's subscription rather than the product meta - which could be edited by the site owner after purchases have happened.
	*
	* @since    1.0.0
	* @param    bool - $expiration_time - The timestamp this All Access Pass should expire.
	* @param    object - $all_access_pass - The All Access Pass object.
	* @return   bool - $expiration_time - The timestamp this All Access Pass should expire.
	*/
	public function sync_expiration_time_with_recurring( $expiration_time, $all_access_pass ){

		// If this All Access pass is set to expire when Recurring does (Sync with Recurring expiration)
		if ( $all_access_pass->duration_unit != 'edd_recurring' ) {
			return $expiration_time;
		}

		// Get the duration variables from the All Access Pass
		$purchase_time = strtotime($all_access_pass->payment->date, current_time( 'timestamp' ) );

		// We'll use the duration number once EDD Recurring supports it.
		// $all_access_duration_number = $all_access_pass->duration_number;

		// Get the duration unit
		$all_access_duration_unit = $all_access_pass->duration_unit;

		// Check if this payment has a subscription attached to it
		$payment_has_subscription = edd_get_payment_meta( $all_access_pass->payment->ID, '_edd_subscription_payment' );

		// Check if this payment is a renewal payment
		$payment_is_renewal = $all_access_pass->payment->status == 'edd_subscription' && ! empty( $all_access_pass->payment->parent_payment ) ? true : false;

		/**
		* If no subscription has been set up with this payment, default it to never expire (as it will never recurr).
		*/
		if ( ! $payment_has_subscription && ! $payment_is_renewal ){

			return 'never';

		}

		if ( $payment_is_renewal ){
			$initial_subscription_payment = $all_access_pass->payment->parent_payment;
		}else{
			$initial_subscription_payment = $all_access_pass->payment->ID;
		}

		// Get all subscriptions attached to this payment
		$subs_db = new EDD_Subscriptions_DB;
		$subs = $subs_db->get_subscriptions( array( 'parent_payment_id' => $initial_subscription_payment, 'order' => 'ASC' ) );

		// Loop through all subscriptions attached to this payment
		foreach( $subs as $sub ){

			// If the product ID attached to the subcription matches the product ID we are checking for expiration
			if ( $sub->product_id == $all_access_pass->download_id ){

				// If this subscription is in trial mode, we'll get the expiration from the trial length
				if ( 'trialling' == $sub->status ){
					// Use the trial period saved into this subscription
					$all_access_duration_unit = $sub->trial_period;
				}elseif( 'expired' == $sub->status && 0 == $sub->times_billed && ! empty( $sub->trial_period ) ){
					// If this subscription was a trial that was never renewed, use the trial period saved into this subscription
					$all_access_duration_unit = $sub->trial_period;
				}else{
					// Use the period saved into this subscription
					$all_access_duration_unit = $sub->period;
				}
			}
		}

		// Edd recurring doesn't have a duration number so this is always 1 (Example 1 day, 1 month, 1 year etc). If they ever add that, fetch it here.
		$all_access_duration_number = 1;

		//Add the amount of time the payment is valid for to the purchase time to get the expiration time
		$expiration_time = strtotime( '+' . $all_access_duration_number . ' ' . $all_access_duration_unit, $purchase_time );

		if ( empty( $expiration_time ) ){
			$expiration_time = 'Failure in recurring hook 2. Expiration was empty. Values were Duration Number:' . $all_access_duration_number . ' Duration Unit: '. $all_access_duration_unit . 'Purchase Time:' . $purchase_time;
		}

		return $expiration_time;

	}

	/**
	* Show the right string for the All Access Pass if the duration is set to "Sync With Recurring"
	*
	* @since    1.0.0
	* @param    string - $assembled_string - The All Access duration string shown to the user. For example, "1 year".
	* @param    object - $all_access_pass - The Al Access Pass Object
	* @return   bool - $is_active - whether this All Access pass is active or not.
	*/
	public function duration_string( $assembled_string, $all_access_pass ){

		if ( $all_access_pass->duration_unit != 'edd_recurring' ){
			return $assembled_string;
		}

		if ( empty( $all_access_pass->payment->parent_payment ) ){
			$parent_payment_id = $all_access_pass->payment->ID;
		}else{
			$parent_payment_id = $all_access_pass->payment->parent_payment;
		}

		// Get all subscriptions attached to this payment
		$subs_db = new EDD_Subscriptions_DB;
		$subs = $subs_db->get_subscriptions( array( 'parent_payment_id' => $parent_payment_id, 'order' => 'ASC' ) );

		// Loop through all subscriptions attached to this payment
		foreach( $subs as $sub ){

			// If the product ID attached to the subcription matches the product ID we are checking for expiration
			if ( $sub->product_id == $all_access_pass->download_id ){
				// Use the period saved into this subcrition
				$all_access_duration_unit = $sub->period;
			}
		}

		return '1 ' . $all_access_duration_unit . ' ' . __( '(recurring)', 'edd-all-access' );

	}

	/**
	* This is fired directly after a subscription has been renewed. Here we hook into that to expire the old All Access Pass and activate the new.
	*
	* @since    1.0.0
	* @param    int      $subscription_id - The ID of the subscription that was renewed.
	* @param    string   $subscription_expiration - The expiration date of the subscription in format Y-m-d H:i:s'
	* @param    object   $edd_subscription - The Al Access Pass Object
	* @return   void
	*/
	public function check_expirations_post_renew( $subscription_id, $subscription_expiration, $edd_subscription ){

		// Get the original payment id
		$original_payment_id = $edd_subscription->get_original_payment_id();

		// Set up the payment object from the original payment so we can see what was purchased
		$parent_payment = new EDD_Payment( $original_payment_id );

		// Set default price id
		$price_id = 0;

		// We need to get the price id in order to find the All Access Pass. Loop through all items in this cart until we find the product being renewed.
		foreach ( $parent_payment->cart_details as $cart_item ) {
			if ( (int) $edd_subscription->product_id === (int) $cart_item['id'] ) {
				$price_id = $cart_item['item_number']['options']['price_id'];
				break;
			}
		}

		// Get all renewal payments attached to this renewed subscription.
		$renewal_payments = $edd_subscription->get_child_payments();

		// Get the second-newest renewal payment because we need to expire the All Access Pass attached to it
		$expired_payment = isset( $renewal_payments[1] ) ? $renewal_payments[1] : false;

		if ( ! $expired_payment ){
			$expired_payment_id = $edd_subscription->get_original_payment_id();
		}else{
			$expired_payment_id = $expired_payment->ID;
		}

		// Set up the All Access Pass object for the expired payment
		$old_all_access_pass = new EDD_All_Access_Pass( $expired_payment_id, $edd_subscription->product_id, $price_id );

		// Attempt to expire the old All Access Pass
		$old_all_access_pass->maybe_expire();

	}

	/**
	 * Prevent manual early renewal purchases of All Access if there is an active Subscription for it.
	 *
	 * @since       1.0.0
	 * @param       array $valid_data The values relating to displaying the Purchase Button
	 * @param       array $post_data The values relating to displaying the Purchase Button
	 * @return      void
	 */
	public function prevent_manual_renewals( $valid_data, $post_data ) {

		// Get the currently logged-in customer
		$customer = new EDD_Recurring_Subscriber( get_current_user_id(), true );

		// Get all of the All Access enabled products
		$all_access_products = edd_all_access_get_all_access_downloads();

		$cart_contents = edd_get_cart_contents();

		// Loop through each item in the cart to check if it is an All Access with an active Subscription
		foreach( $cart_contents as $cart_key => $item ){

			$download_id = $item['id'];
			$price_id = isset( $item['options']['price_id'] ) ? (int) $item['options']['price_id'] : 0;

			// If this download is an All Access Product
			if ( in_array( $download_id, $all_access_products ) ){

				// Get the currently logged-in customer
				$customer = new EDD_Recurring_Subscriber( get_current_user_id(), true );

				// Check if it has an active subscription
				$subs = $customer->get_subscriptions( $download_id );

				if ( $subs ) {

					foreach ( $subs as $sub ) {

						// If this is not expired and has a status of active or trialling (we allow cancelled ones to be renewed if they are not expired)
						if ( ! $sub->is_expired() && ( $sub->status == 'active' || $sub->status == 'trialling' ) && edd_is_payment_complete( $sub->parent_payment_id ) ) {

							// Prevent the renewal as it will just cause the customer to double pay for no reason.
							edd_set_error( 'edd_all_access_subscription_already_exists', sprintf( __( 'You already have an active subscription for %s.', 'edd-all-access' ), get_the_title( $download_id ) ) );
						}
					}
				}
			}
		}
	}
}
