<?php
/**
 * Integration functions to make All Access compatible with EDD Software Licenses
 *
 * @package     EDD\EDDAllAccess\Functions
 * @since       1.0.0
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

/**
 * Integrates EDD All Access with the EDD Software Licensing extension
 *
 * @since 1.0.0
 */
class EDD_All_Access_Software_Licensing {

	/**
	 * Get things started
	 *
	 * @since  1.0.0
	 * @return void
	 */
	public function __construct() {

		if ( ! class_exists( 'EDD_Software_Licensing' ) ) {
			return;
		}

		add_action( 'admin_notices', array( $this, 'edd_all_access_sl_too_old_notice' ) );
		add_filter( 'edd_sl_check_item_name', array( $this, 'edd_all_access_sl_name_matches' ), 10, 4 );
		add_filter( 'edd_sl_id_license_match', array( $this, 'edd_all_access_sl_id_matches' ), 10, 4 );
		add_filter( 'edd_sl_force_check_by_name', array( $this, 'edd_all_access_sl_force_check_by_name' ) );
		add_action( 'template_redirect', array( $this, 'edd_all_access_sync_renewals' ) );
		add_action( 'edd_sl_license_upgraded', array( $this, 'edd_all_access_sl_upgrade' ), 10, 2 );
		add_filter( 'edd_sl_allow_bundle_activation', array( $this, 'allow_bundle_activations' ), 10, 2 );

	}

	/**
	 * Make sure we are at the minimum version of Software Licensing - which is 3.5.
	 *
	 * @since       1.0.0
	 */
	public function edd_all_access_sl_too_old_notice(){

		if ( defined( 'EDD_SL_VERSION' ) && version_compare( EDD_SL_VERSION, '3.5.18', '<' ) ){
			?>
			<div class="notice notice-error">
				<p><?php echo __( 'EDD All Access: Your version of EDD Software Licensing must be updated to version 3.5.18 or later to use the All Access extension in conjunction with Software Licensing.', 'edd-all-access' ); ?></p>
			</div>
			<?php
		}
	}

	/**
	 * Software Licensing does not allow bundle activations to happen. In the case where a bundle-license is also an All Access product
	 *
	 * @since    1.0.2
	 * @param    bool	$allow_bundle_activation
	 * @param    obj	$license
	 * @return   bool	$allow_bundle_activation
	 */
	public function allow_bundle_activations( $allow_bundle_activation, $license ){

		// Get all of the ALl Access products in this store
		$all_access_products = edd_all_access_get_all_access_downloads();

		// Loop through each All Access product
		foreach( $all_access_products as $all_access_product_id ){

			// If the bundled license is an All Access product
			if( $license->download->id == $all_access_product_id ){

				// Allow this bundle license to be activated
				return true;
			}
		}

		return $allow_bundle_activation;

	}

	/**
	 * When Software Licensing is checking if the passed-in title matches the title attached to the passed-in license in our Store,
	 * Check if the license's product-title is an All Access product which includes the product trying to be accessed/downloaded. If it does, tell
	 * Software Licensing the titles match up. In this way, we can "trick" Software Licensing into accepting a license other than one for the actual product.
	 * In this case, it allows for a "master" license key to be used for products it isn't actually for (outside of the All Access extension).
	 *
	 * @since    1.0.0
	 * @param    bool	$item_name_matches
	 * @param    int	$download_id
	 * @param    string	$item_name
	 * @param    object	$license
	 * @return   bool	$item_name_matches
	 */
	public function edd_all_access_sl_name_matches( $item_name_matches, $download_id, $item_name, $license ){

		// If no license was passed, they might be running an older version of Software Licensing.
		if ( ! isset( $license ) || empty( $license ) ) {
			return $item_name_matches;
		}

		// Decode the item name since it came from a URL
		$item_name_decode = urldecode( $item_name );

		// Get a WP Post object using that decoded title
		$post_with_title_passed_in = get_page_by_title( $item_name_decode, 'OBJECT', 'download' );

		// If we didn't find a post using the decoded title, double check it for special characters not covered by urldecode.
		if ( ! $post_with_title_passed_in ){

 			//Do a double check using rawurldecode in case the plugn author used a + sign in the Download's title and used rawurlencode to send the title to us here.
			$item_name_raw_decode = rawurldecode( $item_name );

			$post_with_title_passed_in = get_page_by_title( $item_name_raw_decode, 'OBJECT', 'download' );

			// If no WP post objects were found using the passed-in item name, return the incoming value which we got from the filter.
			if ( ! $post_with_title_passed_in ){
				return $item_name_matches;
			}
		}

		$all_access_check = edd_all_access_check( array(
			'download_id' => $post_with_title_passed_in->ID,
			'price_id' => NULL,
			'customer_id' => $license->customer_id,
			'require_login' => false
		 ) );

		// If the customer attached to the license has an All Access pass which includes the desired product
		if ( isset( $all_access_check['success'] ) && $all_access_check['success'] ){
			return true;
		}

		// Otherwise, return the unchanged value that we got from the filter hook
		return $item_name_matches;

	}

	/**
	 * When Software Licensing is checking if the passed-in ID matches the ID attached to the passed-in license in our Store,
	 * Check if the license's ID is an All Access product which includes the product trying to be accessed/downloaded. If it does, tell
	 * Software Licensing the IDs match up. In this way, we can "trick" Software Licensing into accepting a license other than one for the actual product.
	 * In this case, it allows for a "master" license key to be used for products it isn't actually for (outside of the All Access extension).
	 *
	 * @since    1.0.0
	 * @param    bool	$license_match
	 * @param    int	$download_id
	 * @param    int	$license_download
	 * @param    string	$license_key
	 * @return   bool	$license_match
	 */
	public function edd_all_access_sl_id_matches( $license_match, $download_id, $license_download, $license_key ){

		// Check if the license is an All Access license which includes the desired product

		// Get the customer attached to the license
		$license = edd_software_licensing()->get_license( $license_key, true );

		$all_access_check = edd_all_access_check( array(
			'download_id' => $download_id,
			'price_id' => NULL,
			'customer_id' => $license->customer_id,
			'require_login' => false
		 ) );

		// If the customer attached to the license has an All Access pass which includes the desired product
		if ( isset( $all_access_check['success'] ) && $all_access_check['success'] ){
			return true;
		}

		// Otherwise, return the unchanged value that we got from the filter hook
		return $license_match;
	}

	/**
	 * Tell Software Licensing if we should check for new version updates using the passed-in license or the passed-in name.
	 * Because the All Access license isn't the product we are hoping to check for updates, we want to force Software Licensing to check using the name.
	 *
	 * @since    1.0.0
	 * @param    bool	$check_by_name_first Whether we should fetch update data using the passed-in name or using the passed-in license.
	 * @return   bool	$check_by_name_first We want to fetch update data using the passed-in name.
	 */
	public function edd_all_access_sl_force_check_by_name( $check_by_name_first ){
		return true;
	}

	/**
	 * This fires directly after a license is upgraded. We use it here to update the All Access Pass customer meta
	 * of both the old payment and the new payment.
	 * The All Access Pass attached to the old_payment will get a flag letting it know it is an old/prior All Access Pass.
	 * The All Access Pass attached to the new_payment will get an array containing all preliminary payments by taking them from the old All Access Pass. .
	 *
	 * @since    1.0.0
	 * @param    str	$license_id A timestamp of the time of upgrade payment
	 * @param    arr	$args An array of settings relating to the upgrade
	 * @return   void
	 */
	public function edd_all_access_sl_upgrade( $license_id, $args ){

		$default_args = array(
			'payment_id'       => 0,
			'old_payment_id'   => 0,
			'download_id'      => 0,
			'old_download_id'  => 0,
			'old_price_id'     => false,
			'upgrade_id'       => 0,
			'upgrade_price_id' => false
		);

		$args = wp_parse_args( $args, $default_args );

		$old_all_access_pass = new EDD_All_Access_Pass( $args['old_payment_id'], $args['old_download_id'], $args['old_price_id'] );
		$new_all_access_pass = new EDD_All_Access_Pass( $args['payment_id'], $args['download_id'], $args['upgrade_price_id'] );

		// Upgrade the old All Access Pass to the new All Access Pass
		$old_all_access_pass->do_upgrade( $new_all_access_pass );

	}

	/**
	 * If an All Access Pass is being renewed and it is licensed, make sure the license is "renewed" in Software Licensing - as opposed to created a new license.
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function edd_all_access_sync_renewals(){

		// If license renewals are not allowed, skip this entirely
		if ( ! edd_sl_renewals_allowed() ){
			return false;
		}

		// Check everything in the cart to see if any are Licensed All Access Renewals
		$cart_contents = edd_get_cart_contents();

		if ( empty( $cart_contents ) ){
			return false;
		}

		$current_user_id = get_current_user_id();
		$customer = new EDD_Customer( $current_user_id, true );

		// If this is not an existing customer, this is definitely not a renewal, so we can leave here now.
		if ( $customer->id == 0 ){
			return false;
		}

		// Get this customer's All Access Passes (if any)
		$customers_all_access_passes = $customer->get_meta( 'all_access_passes' );

		if ( empty( $customers_all_access_passes ) ){
			return false;
		}

		foreach( $cart_contents as $cart_key => $cart_item ){

			$download_id = $cart_item['id'];
			$price_id = isset( $cart_item['options']['price_id'] ) ? (int) $cart_item['options']['price_id'] : 0;

			// If this product is licensed
			$license = new EDD_SL_Download( $download_id );

			// Skip over products without licensing enabled
			if ( ! $license->licensing_enabled() ){
				continue;
			}

			// If the customer has purchased this All Access Pass Before, this is a renewal
			if ( array_key_exists( $download_id . '_' . $price_id, $customers_all_access_passes ) ){

				$previous_payment_id = $customers_all_access_passes[$download_id . '_' . $price_id]['payment_id'];

				// Get the previous license key
				$license = edd_software_licensing()->get_license_by_purchase( $previous_payment_id, $download_id );

				// If no license was found, skip this cart item
				if ( empty( $license ) ){
					continue;
				}

				$added = edd_sl_add_renewal_to_cart( sanitize_text_field( $license->key ), true );

				if( $added && ! is_wp_error( $added ) ) {

					$redirect = edd_get_checkout_uri();

				} else {

					$code     = $added->get_error_code();
					$message  = $added->get_error_message();
					$redirect = add_query_arg( array( 'edd-sl-error' => $code, 'message' => urlencode( $message ) ), edd_get_checkout_uri() );

				}
			}
		}
	}
}
