<?php

class Tests_EDD_All_Access_Class extends WP_UnitTestCase {
	protected $object;

	public static $_user;
	public static $_author;
	public static $_all_access_pass;
	public static $_customer;

	// Completed payment variables
	public static $_payment_id;
	public static $_payment;
	public static $_download_id;
	public static $_download;

	// Renewal payment variables
	public static $_renewal_payment_id;
	public static $_renewal_payment;
	public static $_renewal_download_id;
	public static $_renewal_download;

	// Pending payment variables
	public static $_pending_payment_id;
	public static $_pending_payment;
	public static $_pending_download_id;
	public static $_pending_download;

	// Non AA payment variables
	public static $_non_aa_payment_id;
	public static $_non_aa_payment;
	public static $_non_aa_download_id;
	public static $_non_aa_download;

	public static function wpSetUpBeforeClass() {

		edd_run_install();

		// Let's make some default users for later
		// `Author` can be used to make products that have commissions assigned to him
		$author = array(
			'user_login'  =>  'author',
			'roles'       =>  array( 'author' ),
			'user_pass'   => NULL,
		);

		wp_insert_user( $author );

		// Create `Subscriber` user
		$subscriber = array(
			'user_login'  =>  'subscriber',
			'roles'       =>  array( 'subscriber' ),
			'user_pass'   => NULL,
		);

		wp_insert_user( $subscriber );

		// Set up a payment containing an All Access Product
		self::$_payment_id  = EDD_Helper_Payment::create_simple_payment();
		self::$_payment     = new EDD_Payment( self::$_payment_id );
		self::$_download_id = self::$_payment->downloads[ 0 ][ 'id' ];
		self::$_download    = new EDD_Download( self::$_download_id );
		self::$_user        = get_user_by( 'login', 'subscriber' );
		self::$_author      = get_user_by( 'login', 'author' );

		// Create a non All Access product
		self::$_non_aa_download = EDD_Helper_Download::create_simple_download();
		self::$_non_aa_download_id = self::$_non_aa_download->ID;

		// Create a renewal payment for the All Access Product
		self::$_renewal_payment_id = EDD_Helper_Payment::create_simple_payment( array( 'download_id' => self::$_download_id ) );
		self::$_renewal_payment = new EDD_Payment( self::$_renewal_payment_id );

		// Create a pending payment containing an All Access Product
		self::$_pending_payment_id  = EDD_Helper_Payment::create_simple_payment();
		self::$_pending_payment = new EDD_Payment( self::$_pending_payment_id );

		// Create a payment that does not contain an All Access Product
		self::$_non_aa_payment_id  = EDD_Helper_Payment::create_simple_payment();
		self::$_non_aa_payment = new EDD_Payment( self::$_pending_payment_id );

		// Set the All Access Pass's meta data
		$all_access_product_config = array(
			'all_access_duration_number' => 1,
			'all_access_duration_unit' => 'year',
			'download_limit' => 10,
			'download_limit_time_period' => 'per_day',
			'all_access_categories' => array( 'all' ),
			'number_of_price_ids' => 0,
			'included_price_ids' => 0,
		);

		update_post_meta( self::$_download_id, '_edd_all_access_enabled', true );
		update_post_meta( self::$_download_id, '_edd_all_access_settings', $all_access_product_config );
		update_option( 'edd_all_access_products', array( self::$_download_id ) );

		self::$_payment->status = 'publish';
		self::$_payment->save();
		// Refresh the payment now that the status is publish
		self::$_payment = new EDD_Payment( self::$_payment_id );

		self::$_pending_payment->status = 'pending';
		self::$_pending_payment->save();
		// Refresh the pending payment now that the status is pending
		self::$_pending_payment = new EDD_Payment( self::$_pending_payment_id );

		self::$_renewal_payment->status = 'publish';
		self::$_renewal_payment->save();
		// Refresh the renewal payment now that the status is publish
		self::$_renewal_payment = new EDD_Payment( self::$_renewal_payment_id );

		self::$_non_aa_payment->status = 'publish';
		self::$_non_aa_payment->save();
		// Refresh the non aa payment now that the status is publish
		self::$_non_aa_payment = new EDD_Payment( self::$_non_aa_payment_id );

		// Instantiate a new All Access Pass object
		self::$_all_access_pass = new EDD_All_Access_Pass( self::$_payment->ID, self::$_download_id, 1 );

		self::$_customer = new EDD_Customer( self::$_payment->customer_id );
	}

	public function setUp() {
		parent::setUp();
	}

	public function tearDown() {
		parent::tearDown();
	}

	/**
	 * Test the All Access Pass status
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_statuses() {

		// Test for invalid because the download id is blank
		$invalid_pass_1 = new EDD_All_Access_Pass( 0, 0, 0 );
		$this->assertSame( 'invalid', $invalid_pass_1->status );

		// Test for invalid because the payment Id does not exist
		$invalid_pass_2 = new EDD_All_Access_Pass( self::$_payment->ID + 1, self::$_download_id, 1 );
		$this->assertSame( 'invalid', $invalid_pass_2->status );

		// Test for invalid because the payment status is not one of the valid ones
		$invalid_pass_3 = new EDD_All_Access_Pass( self::$_pending_payment->ID + 1, self::$_download_id, 1 );
		$this->assertSame( 'invalid', $invalid_pass_3->status );

		// Test for invalid because the download ID does not contain an All Access product
		$invalid_pass_4 = new EDD_All_Access_Pass( self::$_non_aa_payment_id + 1, self::$_non_aa_download_id, 1 );
		$this->assertSame( 'invalid', $invalid_pass_4->status );

		// Test for upcoming because the $purchased_aa_download_key is not yet in the payment meta
		$invalid_pass_5 = new EDD_All_Access_Pass( self::$_renewal_payment_id, self::$_download_id, 1 );
		$this->assertSame( 'upcoming', $invalid_pass_5->status );

		// Test for active All Access Pass
		$invalid_pass_6 = new EDD_All_Access_Pass( self::$_payment_id, self::$_download_id, 1 );
		$this->assertSame( 'active', $invalid_pass_6->status );

		// Now lets expire the current pass so we can test for a renewal


		// Test for renewed All Access Pass
		$invalid_pass_6 = new EDD_All_Access_Pass( self::$_payment_id, self::$_download_id, 1 );
		$this->assertSame( 'active', $invalid_pass_6->status );

		// Test for invalid because the download id is not in the payment
		$invalid_pass_7 = new EDD_All_Access_Pass( self::$_payment->ID, self::$_download_id+1, 1 );
		$this->assertSame( 'invalid', $invalid_pass_7->status );

	}

	/**
	 * Test the edd_all_access_get_all_access_downloads function.
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_get_all_access_downloads() {

		$actual = edd_all_access_get_all_access_downloads();
		$expected = array( self::$_download_id );

		$this->assertSame( $expected, $actual );

	}

	/**
	 * Test the All Access Pass id
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_ID() {
		$this->assertSame( (self::$_payment->ID . '_' . self::$_download_id .'_' . 1 ), self::$_all_access_pass->id );
	}

	/**
	 * Test the All Access Pass id (negative test)
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_ID_false() {
		$this->assertNotEquals( 'This is not the All Access Pass ID', self::$_all_access_pass->id );
	}

	/**
	 * Test the All Access Pass payment object
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_payment_object() {
		$this->assertEquals( self::$_payment, self::$_all_access_pass->payment );
	}

	/**
	 * Test the All Access Pass payment object (negative)
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_payment_object_false() {
		$this->assertNotEquals( '', self::$_all_access_pass->payment );
	}

	/**
	 * Test the All Access Pass Payment ID
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_payment_id() {
		$this->assertSame( self::$_payment->ID, self::$_all_access_pass->payment_id );
	}

	/**
	 * Test the All Access Pass Payment ID (negative)
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_payment_id_false() {
		$this->assertNotEquals( self::$_payment->ID + 1, self::$_all_access_pass->payment_id );
	}

	/**
	 * Test the All Access Pass customer object
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_customer_object() {

		$this->assertEquals( self::$_customer, self::$_all_access_pass->customer );
	}

	/**
	 * Test the All Access Pass customer object (negative)
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_customer_object_false() {
		$this->assertNotEquals( '', self::$_all_access_pass->customer );
	}

	/**
	 * Test the All Access Pass download id
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_download_id() {

		$this->assertSame( self::$_download_id, self::$_all_access_pass->download_id );
	}

	/**
	 * Test the All Access Pass download id (negative)
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	 public function test_all_access_download_id_false() {

 		$this->assertNotEquals( self::$_download_id + 1, self::$_all_access_pass->download_id );
 	}

	/**
	 * Test the All Access Pass price id
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	public function test_all_access_price_id() {

		$this->assertSame( 1, self::$_all_access_pass->price_id );
	}

	/**
	 * Test the All Access Pass price id (negative)
	 *
	 * @since    1.0.0
	 * @param    void
	 * @return   void
	 */
	 public function test_all_access_price_id_false() {

 		$this->assertNotEquals( 2, self::$_all_access_pass->price_id );
 	}

}
